@@ -166,7 +166,7 @@ update_preedit_text(
 		{
 			u_int  count ;
 			IBusAttribute *  attr ;
-			int  is_biwidth = 0 ;
+			int  is_fullwidth = 0 ;
 			int  is_comb = 0 ;
 			int  is_underlined = 0 ;
 			ml_color_t  fg_color = ML_FG_COLOR ;
@@ -209,22 +209,23 @@ update_preedit_text(
 				continue ;
 			}
 
-			if( ch.property & MKF_BIWIDTH)
+			if( ch.property & MKF_FULLWIDTH)
 			{
-				is_biwidth = 1 ;
+				is_fullwidth = 1 ;
 			}
 			else if( ch.property & MKF_AWIDTH)
 			{
 				/* TODO: check col_size_of_width_a */
-				is_biwidth = 1 ;
+				is_fullwidth = 1 ;
 			}
 
 			if( ch.property & MKF_COMBINING)
 			{
 				is_comb = 1 ;
 
-				if( (*syms->ml_char_combine)( p - 1 , ch.ch , ch.size , ch.cs ,
-					is_biwidth , is_comb , fg_color , bg_color , 0 , 1))
+				if( (*syms->ml_char_combine)( p - 1 , mkf_char_to_int(&ch) ,
+					ch.cs , is_fullwidth , is_comb , fg_color , bg_color ,
+					0 , 0 , 1))
 				{
 					continue ;
 				}
@@ -234,10 +235,10 @@ update_preedit_text(
 				 */
 			}
 
-			(*syms->ml_char_set)( p , ch.ch , ch.size , ch.cs ,
-					      is_biwidth , is_comb ,
+			(*syms->ml_char_set)( p , mkf_char_to_int(&ch) , ch.cs ,
+					      is_fullwidth , is_comb ,
 					      fg_color , bg_color ,
-					      0 , 1) ;
+					      0 , 0 , 1) ;
 
 			p ++ ;
 			ibus->im.preedit.filled_len ++ ;
