@@ -220,10 +220,6 @@ set_config(
 			ml_term_set_col_size_of_width_a( nativeObj->term , size) ;
 		}
 	}
-	else if( strcmp( key , "icon_path") == 0)
-	{
-		ml_term_set_icon_path( nativeObj->term , value) ;
-	}
 	else if( strcmp( key , "logging_vt_seq") == 0)
 	{
 		int  flag ;
@@ -233,17 +229,6 @@ set_config(
 			ml_term_set_logging_vt_seq( nativeObj->term , flag) ;
 		}
 	}
-	else if( strcmp( key , "logging_msg") == 0)
-	{
-		if( true_or_false( value) > 0)
-		{
-			kik_set_msg_log_file_name( "mlterm/msg.log") ;
-		}
-		else
-		{
-			kik_set_msg_log_file_name( NULL) ;
-		}
-	}
 	else if( strcmp( key , "use_local_echo") == 0)
 	{
 		int  flag ;
@@ -253,24 +238,6 @@ set_config(
 			ml_term_set_use_local_echo( nativeObj->term , flag) ;
 		}
 	}
-	else if( strcmp( key , "use_alt_buffer") == 0)
-	{
-		int  flag ;
-
-		if( ( flag = true_or_false( value)) != -1)
-		{
-			ml_set_use_alt_buffer( flag) ;
-		}
-	}
-	else if( strcmp( key , "use_ansi_colors") == 0)
-	{
-		int  flag ;
-
-		if( ( flag = true_or_false( value)) != -1)
-		{
-			ml_set_use_ansi_colors( flag) ;
-		}
-	}
 }
 
 static void
@@ -375,10 +342,6 @@ get_config(
 			value = ml_term_get_slave_name( nativeObj->term) ;
 		}
 	}
-	else if( strcmp( key , "icon_path") == 0)
-	{
-		value = ml_term_icon_path( nativeObj->term) ;
-	}
 	else if( strcmp( key , "logging_vt_seq") == 0)
 	{
 		if( ml_term_is_logging_vt_seq( nativeObj->term))
@@ -568,7 +531,7 @@ need_style(
 	if( ml_char_fg_color( ch) != ML_FG_COLOR ||
 	    ml_char_bg_color( ch) != ML_BG_COLOR ||
 	    ml_char_is_underlined( ch) ||
-	    (ml_char_font( ch) & FONT_BOLD) )
+	    (ml_char_font( ch) & (FONT_BOLD|FONT_ITALIC)))
 	{
 		need_style = 2 ;
 	}
@@ -581,7 +544,8 @@ need_style(
 	    ml_char_fg_color( ch) == ml_char_fg_color( prev_ch) &&
 	    ml_char_bg_color( ch) == ml_char_bg_color( prev_ch) &&
 	    ml_char_is_underlined( ch) == ml_char_is_underlined( prev_ch) &&
-	    (ml_char_font( ch) & ml_char_font( prev_ch) & FONT_BOLD) )
+	    (ml_char_font( ch) & (FONT_BOLD|FONT_ITALIC)) ==
+	    (ml_char_font( prev_ch) & (FONT_BOLD|FONT_ITALIC)) )
 	{
 		if( need_style)
 		{
@@ -779,7 +743,7 @@ Java_mlterm_MLTermPty_nativeOpen(
 	{
 		kik_conf_t *  conf ;
 
-		kik_init_prog( "mlterm" , "3.0.11") ;
+		kik_init_prog( "mlterm" , "3.3.0") ;
 		kik_set_sys_conf_dir( CONFIG_PATH) ;
 		kik_locale_init( "") ;
 		kik_sig_child_init() ;
@@ -858,7 +822,7 @@ Java_mlterm_MLTermPty_nativeOpen(
 				kik_str_to_uint( &tab_size , value) ;
 			}
 
-			if( ( value = kik_conf_get_value( conf , "ENCODING")))
+			if( ( value = kik_conf_get_value( conf , "encoding")))
 			{
 				ml_char_encoding_t  e ;
 				if( ( e = ml_get_char_encoding( value)) != ML_UNKNOWN_ENCODING)
@@ -954,6 +918,22 @@ Java_mlterm_MLTermPty_nativeOpen(
 			#endif
 			}
 
+			if( ( value = kik_conf_get_value( conf , "use_alt_buffer")))
+			{
+				if( strcmp( value , "false") == 0)
+				{
+					ml_set_use_alt_buffer( 0) ;
+				}
+			}
+
+			if( ( value = kik_conf_get_value( conf , "use_ansi_colors")))
+			{
+				if( strcmp( value , "false") == 0)
+				{
+					ml_set_use_ansi_colors( 0) ;
+				}
+			}
+
 		#ifdef  USE_LIBSSH2
 			if( ( value = kik_conf_get_value( conf , "ssh_public_key")))
 			{
@@ -983,7 +963,15 @@ Java_mlterm_MLTermPty_nativeOpen(
 			{
 				if( strcmp( value , "true") == 0)
 				{
-					ml_pty_ssh_set_use_x11_forwarding( 1) ;
+					ml_pty_ssh_set_use_x11_forwarding( NULL , 1) ;
+				}
+			}
+
+			if( ( value = kik_conf_get_value( conf , "allow_scp")))
+			{
+				if( strcmp( value , "true") == 0)
+				{
+					ml_set_use_scp( 1) ;
 				}
 			}
 		#endif
@@ -1012,6 +1000,8 @@ Java_mlterm_MLTermPty_nativeOpen(
 
 			kik_conf_delete( conf) ;
 		}
+
+		unicode_policy |= ONLY_USE_UNICODE_BOXDRAW_FONT ;
 	}
 
 	if( ! ( nativeObj = calloc( sizeof( native_obj_t) , 1)))
@@ -1296,6 +1286,13 @@ Java_mlterm_MLTermPty_waitForReading(
 
 	while( 1)
 	{
+	#ifdef  USE_LIBSSH2
+		if( ml_pty_ssh_poll( &read_fds) > 0)
+		{
+			return  JNI_TRUE ;
+		}
+	#endif
+
 		maxfd = 0 ;
 		FD_ZERO( &read_fds) ;
 
@@ -1338,16 +1335,6 @@ Java_mlterm_MLTermPty_waitForReading(
 		}
 	}
 
-#ifdef  USE_LIBSSH2
-	for( count = 0 ; count < num_of_xssh_fds ; count++)
-	{
-		if( FD_ISSET( xssh_fds[count] , &read_fds))
-		{
-			ml_pty_ssh_send_recv_x11( count) ;
-		}
-	}
-#endif
-
 	return  JNI_TRUE ;
 #endif
 }
@@ -1386,6 +1373,16 @@ Java_mlterm_MLTermPty_nativeRead(
 	{
 		int  ret ;
 
+	#ifdef  USE_LIBSSH2
+		u_int  count ;
+		int *  xssh_fds ;
+
+		for( count = ml_pty_ssh_get_x11_fds( &xssh_fds) ; count > 0 ; count--)
+		{
+			ml_pty_ssh_send_recv_x11( count - 1 , 1) ;
+		}
+	#endif
+
 		/* For event listeners of ml_term_t. */
 		nativeObj->env = env ;
 		nativeObj->obj = obj ;
@@ -1553,6 +1550,7 @@ Java_mlterm_MLTermPty_nativeGetRedrawString(
 	static jfieldID  style_bg_pixel ;
 	static jfieldID  style_underline ;
 	static jfieldID  style_bold ;
+	static jfieldID  style_italic ;
 	jobjectArray  array ;
 
 	if( ! nativeObj || ! ((native_obj_t*)nativeObj)->term ||
@@ -1615,7 +1613,7 @@ Java_mlterm_MLTermPty_nativeGetRedrawString(
 			int  ret ;
 
 		#if  0
-			if( ml_char_bytes_equal( line->chars + mod_beg + count , ml_nl_ch()))
+			if( ml_char_code_equal( line->chars + mod_beg + count , ml_nl_ch()))
 			{
 				/* Drawing will collapse, but region.str mustn't contain '\n'. */
 				continue ;
@@ -1666,6 +1664,8 @@ Java_mlterm_MLTermPty_nativeGetRedrawString(
 								"underline" , "Z") ;
 					style_bold = (*env)->GetFieldID( env , style_class ,
 								"bold" , "Z") ;
+					style_italic = (*env)->GetFieldID( env , style_class ,
+								"italic" , "Z") ;
 				}
 
 				styles[num_of_styles++] = (*env)->AllocObject( env , style_class) ;
@@ -1703,6 +1703,11 @@ Java_mlterm_MLTermPty_nativeGetRedrawString(
 					style_bold ,
 					ml_char_font( line->chars + mod_beg + count) & FONT_BOLD ?
 						JNI_TRUE : JNI_FALSE) ;
+
+				(*env)->SetBooleanField( env , styles[num_of_styles - 1] ,
+					style_italic ,
+					ml_char_font( line->chars + mod_beg + count) &
+						FONT_ITALIC ? JNI_TRUE : JNI_FALSE) ;
 			}
 
 			redraw_len += len ;
