@@ -14,6 +14,7 @@
 #include  <kiklib/kik_unistd.h>		/* kik_usleep */
 #include  <kiklib/kik_dialog.h>
 #include  <kiklib/kik_net.h>		/* getaddrinfo/socket/connect/sockaddr_un */
+#include  <kiklib/kik_conf_io.h>	/* kik_get_user_rc_path */
 
 #if  ! defined(USE_WIN32API) && defined(HAVE_PTHREAD)
 #include  <pthread.h>
@@ -56,9 +57,16 @@ typedef struct ssh_session
 	LIBSSH2_SESSION *  obj ;
 	int  sock ;
 
+	int  use_x11_forwarding ;
+
 	int  doing_scp ;
 
-	u_int  ref_count ;
+	LIBSSH2_CHANNEL **  pty_channels ;
+	u_int  num_of_ptys ;
+
+	int *  x11_fds ;
+	LIBSSH2_CHANNEL **  x11_channels ;
+	u_int  num_of_x11 ;
 
 } ssh_session_t ;
 
@@ -82,22 +90,6 @@ typedef struct scp
 
 } scp_t ;
 
-#ifndef  USE_WIN32API
-typedef struct x11_fd
-{
-	int  display ;
-	int  channel ;
-
-} x11_fd_t ;
-
-typedef struct x11_channel
-{
-	LIBSSH2_CHANNEL *  channel ;
-	ssh_session_t *  session ;
-
-} x11_channel_t ;
-#endif	/* USE_WIN32API */
-
 
 /* --- static variables --- */
 
@@ -108,6 +100,7 @@ static u_int  num_of_sessions = 0 ;
 
 #ifdef  USE_WIN32API
 static HANDLE  rd_ev ;
+static DWORD  main_tid ;
 #endif
 
 static const char *  cipher_list ;
@@ -116,12 +109,9 @@ static u_int  keepalive_msec ;
 static u_int  keepalive_msec_left ;
 
 static int  use_x11_forwarding ;
-#ifndef  USE_WIN32API
 static int  display_port = -1 ;
-static x11_fd_t  x11_fds[10] ;
-static u_int  num_of_x11_fds ;
-static x11_channel_t  x11_channels[10] ;
-#endif
+
+static int  auth_agent_is_available ;
 
 
 /* --- static functions --- */
@@ -133,7 +123,6 @@ wait_pty_read(
 	LPVOID thr_param
   	)
 {
-	ml_pty_t *  pty = (ml_pty_t*)thr_param ;
 	u_int  count ;
 	struct timeval  tval ;
 	fd_set  read_fds ;
@@ -153,19 +142,28 @@ wait_pty_read(
 
 		for( count = 0 ; count < num_of_sessions ; count++)
 		{
+			u_int  count2 ;
+
 			FD_SET( sessions[count]->sock , &read_fds) ;
 			if( sessions[count]->sock > maxfd)
 			{
 				maxfd = sessions[count]->sock ;
 			}
+
+			for( count2 = 0 ; count2 < sessions[count]->num_of_x11 ; count2++)
+			{
+				FD_SET( sessions[count]->x11_fds[count2] , &read_fds) ;
+				if( sessions[count]->x11_fds[count2] > maxfd)
+				{
+					maxfd = sessions[count]->x11_fds[count2] ;
+				}
+			}
 		}
 
 		if( select( maxfd + 1 , &read_fds , NULL , NULL , &tval) > 0)
 		{
-			if( pty->pty_listener && pty->pty_listener->read_ready)
-			{
-				(*pty->pty_listener->read_ready)( pty->pty_listener->self) ;
-			}
+			/* Exit GetMessage() in x_display_receive_next_event(). */
+			PostThreadMessage( main_tid , WM_APP , 0 , 0) ;
 
 			WaitForSingleObject( rd_ev, INFINITE) ;
 		}
@@ -192,7 +190,6 @@ wait_pty_read(
 
 #endif	/* USE_WIN32API */
 
-
 static void
 kbd_callback(
 	const char *  name ,
@@ -222,6 +219,9 @@ kbd_callback(
 #define  HAVE_GETADDRINFO
 #endif
 
+static void  x11_callback( LIBSSH2_SESSION *  session , LIBSSH2_CHANNEL *  channel ,
+	char *  shost , int  sport , void **  abstract) ;
+
 /*
  * Return session which is blocking mode.
  */
@@ -255,13 +255,12 @@ ssh_connect(
 
 	if( ( session = ml_search_ssh_session( host , port , user)))
 	{
-		session->ref_count ++ ;
 		libssh2_session_set_blocking( session->obj , 1) ;
 
 		return  session ;
 	}
 
-	if( ! ( session = malloc( sizeof(ssh_session_t))))
+	if( ! ( session = calloc( 1 , sizeof(ssh_session_t))))
 	{
 		return  NULL ;
 	}
@@ -373,8 +372,6 @@ ssh_connect(
 
 	libssh2_session_set_blocking( session->obj , 1) ;
 
-	libssh2_session_flag( session->obj , LIBSSH2_FLAG_COMPRESS , 1) ;
-
 	if( cipher_list)
 	{
 		libssh2_session_method_pref( session->obj ,
@@ -383,10 +380,30 @@ ssh_connect(
 			LIBSSH2_METHOD_CRYPT_SC , cipher_list) ;
 	}
 
-	if( libssh2_session_startup( session->obj , session->sock) != 0)
+	/* LIBSSH2_FLAG_COMPRESS doesn't work with X11 forwarding. */
+	libssh2_session_callback_set( session->obj ,
+			LIBSSH2_CALLBACK_X11 , x11_callback) ;
+
+#ifndef  LIBSSH2_FIX_DECOMPRESS_BUG
+	/*
+	 * XXX
+	 * libssh2 1.4.3 fails to decompress zipped packets and breaks X11
+	 * forwarding.
+	 * Camellia branch of http://bitbucket.org/arakiken/libssh2/ fixes this bug
+	 * and defines LIBSSH2_FIX_DECOMPRESS_BUG macro.
+	 */
+	if( ! use_x11_forwarding)
+#endif
+	{
+		libssh2_session_flag( session->obj , LIBSSH2_FLAG_COMPRESS , 1) ;
+	}
+
+	session->use_x11_forwarding = use_x11_forwarding ;
+
+	if( libssh2_session_handshake( session->obj , session->sock) != 0)
 	{
 	#ifdef  DEBUG
-		kik_debug_printf( KIK_DEBUG_TAG " libssh2_session_startup failed.\n") ;
+		kik_debug_printf( KIK_DEBUG_TAG " libssh2_session_handshake failed.\n") ;
 	#endif
 
 		goto  error4 ;
@@ -477,6 +494,55 @@ ssh_connect(
 	{
 		char *  home ;
 		char *  p ;
+		LIBSSH2_AGENT *  agent ;
+
+	#if  defined(__CYGWIN__)
+		if( ! getenv( "SSH_AUTH_SOCK"))
+		{
+			/*
+			 * libssh2_agent_* functions instabilize libssh2 in cygwin, so
+			 * do nothing if SSH_AUTH_SOCK is not defined.
+			 */
+		}
+		else
+	#endif
+		if( ( agent = libssh2_agent_init( session->obj)))
+		{
+			if( libssh2_agent_connect( agent) == 0)
+			{
+				if( libssh2_agent_list_identities( agent) == 0)
+				{
+					struct libssh2_agent_publickey *  ident ;
+					struct libssh2_agent_publickey *  prev_ident ;
+
+					prev_ident = NULL ;
+					while( libssh2_agent_get_identity( agent ,
+								&ident , prev_ident) == 0)
+					{
+						if( libssh2_agent_userauth( agent ,
+								user , ident) == 0)
+						{
+							libssh2_agent_disconnect( agent) ;
+							libssh2_agent_free( agent) ;
+
+							auth_agent_is_available = 1 ;
+
+							goto  pubkey_success ;
+						}
+
+						prev_ident = ident ;
+					}
+				}
+
+				libssh2_agent_disconnect( agent) ;
+			}
+
+			libssh2_agent_free( agent) ;
+		}
+
+	#ifdef  DEBUG
+		kik_debug_printf( KIK_DEBUG_TAG " Unable to use ssh-agent.\n") ;
+	#endif
 
 		if( ( home = kik_get_home_dir()) &&
 		    ( ( p = alloca( strlen(home) * 2 + 38))) )
@@ -528,6 +594,7 @@ ssh_connect(
 		if( libssh2_userauth_publickey_fromfile( session->obj , user ,
 				pubkey , privkey , pass) == 0)
 		{
+		pubkey_success:
 			kik_msg_printf( "Authentication by public key succeeded.\n") ;
 
 			auth_success = 1 ;
@@ -586,16 +653,22 @@ ssh_connect(
 	#endif
 	}
 
-	if( ! ( sessions = realloc( sessions , sizeof(ssh_session_t) * (++num_of_sessions))))
 	{
-		goto  error4 ;
+		void *  p ;
+
+		if( ! ( p = realloc( sessions , sizeof(ssh_session_t) * (num_of_sessions + 1))))
+		{
+			goto  error4 ;
+		}
+
+		sessions = p ;
 	}
 
 	session->host = strdup( host) ;
 	session->port = strdup( port) ;
 	session->user = strdup( user) ;
-	session->ref_count = 1 ;
-	sessions[num_of_sessions - 1] = session ;
+
+	sessions[num_of_sessions ++] = session ;
 
 	return  session ;
 
@@ -618,6 +691,11 @@ error1:
 	return  NULL ;
 }
 
+static void  close_x11( ssh_session_t *  session , int  idx) ;
+
+/*
+ * Call with in blocking mode.
+ */
 static int
 ssh_disconnect(
 	ssh_session_t *  session
@@ -625,12 +703,12 @@ ssh_disconnect(
 {
 	u_int  count ;
 
-	if( -- session->ref_count > 0)
+	if( session->num_of_ptys > 0)
 	{
 		/* In case this function is called from ml_pty_new. */
 		libssh2_session_set_blocking( session->obj , 0) ;
 
-		return  1 ;
+		return  0 ;
 	}
 
 	for( count = 0 ; count < num_of_sessions ; count++)
@@ -649,6 +727,11 @@ ssh_disconnect(
 		}
 	}
 
+	for( count = session->num_of_x11 ; count > 0 ; count--)
+	{
+		close_x11( session , count - 1) ;
+	}
+
 	libssh2_session_disconnect( session->obj , "Normal shutdown, Thank you for playing") ;
 	libssh2_session_free( session->obj) ;
 	closesocket( session->sock) ;
@@ -661,6 +744,9 @@ ssh_disconnect(
 	free( session->host) ;
 	free( session->port) ;
 	free( session->user) ;
+	free( session->pty_channels) ;
+	free( session->x11_fds) ;
+	free( session->x11_channels) ;
 	free( session) ;
 
 #ifdef  DEBUG
@@ -675,9 +761,25 @@ final(
 	ml_pty_t *  pty
 	)
 {
-	libssh2_session_set_blocking( ((ml_pty_ssh_t*)pty)->session->obj , 1) ;
+	ssh_session_t *  session ;
+	u_int  count ;
+
+	session = ((ml_pty_ssh_t*)pty)->session ;
+
+	libssh2_session_set_blocking( session->obj , 1) ;
+
+	for( count = 0 ; count < session->num_of_ptys ; count++)
+	{
+		if( session->pty_channels[count] == ((ml_pty_ssh_t*)pty)->channel)
+		{
+			session->pty_channels[count] =
+				session->pty_channels[--session->num_of_ptys] ;
+			break ;
+		}
+	}
+
 	libssh2_channel_free( ((ml_pty_ssh_t*)pty)->channel) ;
-	ssh_disconnect( ((ml_pty_ssh_t*)pty)->session) ;
+	ssh_disconnect( session) ;
 
 	return  1 ;
 }
@@ -1065,12 +1167,8 @@ scp_thread(
 			ml_write_to_pty( &scp->pty_ssh->pty , "|\r|" , 3) ;
 
 		#ifdef  USE_WIN32API
-			if( scp->pty_ssh->pty.pty_listener &&
-			    scp->pty_ssh->pty.pty_listener->read_ready)
-			{
-				(*scp->pty_ssh->pty.pty_listener->read_ready)(
-					scp->pty_ssh->pty.pty_listener->self) ;
-			}
+			/* Exit GetMessage() in x_display_receive_next_event(). */
+			PostThreadMessage( main_tid , WM_APP , 0 , 0) ;
 		#endif
 		}
 	}
@@ -1100,7 +1198,9 @@ scp_thread(
 }
 
 
-#ifndef  USE_WIN32API
+#if  0
+#define  TRUSTED
+#endif
 
 static int
 setup_x11(
@@ -1110,15 +1210,20 @@ setup_x11(
 	char *  display ;
 	char *  display_port_str ;
 	char *  p ;
-	char *  cmd ;
-	char  line[512] ;
 	char *  proto ;
 	char *  data ;
+#ifndef  USE_WIN32API
+	char *  cmd ;
 	FILE *  fp ;
+	char  line[512] ;
+#ifndef  TRUSTED
+	char *  xauth_file ;
+#endif
+#endif
 
 	if( ! ( display = getenv( "DISPLAY")))
 	{
-		return  0 ;
+		display = ":0.0" ;
 	}
 
 	if( strncmp( display , "unix:" , 5) == 0)
@@ -1149,33 +1254,55 @@ setup_x11(
 	proto = NULL ;
 	data = NULL ;
 
-	if( ( cmd = alloca( 39 + strlen( display) + 1)))
+#ifndef  USE_WIN32API
+#ifdef  TRUSTED
+	if( ( cmd = alloca( 24 + strlen( display) + 1)))
 	{
 		sprintf( cmd , "xauth list %s 2> /dev/null" , display) ;
-		if( ( fp = popen( cmd , "r")))
+#else
+	if( ( xauth_file = kik_get_user_rc_path( "mlterm/xauthfile")))
+	{
+		if( ( cmd = alloca( 61 + strlen(xauth_file) + strlen( display) + 1)))
 		{
-			if( fgets( line , sizeof(line) , fp))
+			sprintf( cmd , "xauth -f %s generate %s MIT-MAGIC-COOKIE-1 "
+				       "untrusted 2> /dev/null" , xauth_file , display) ;
+			system( cmd) ;
+			sprintf( cmd , "xauth -f %s list %s 2> /dev/null" ,
+					xauth_file , display) ;
+#endif
+
+			if( ( fp = popen( cmd , "r")))
 			{
-				if( ( proto = strchr( line , ' ')))
+				if( fgets( line , sizeof(line) , fp))
 				{
-					proto += 2 ;
-
-					if( ( data = strchr( proto , ' ')))
+					if( ( proto = strchr( line , ' ')))
 					{
-						*data = '\0' ;
-						data += 2 ;
+						proto += 2 ;
 
-						if( ( p = strchr( data , '\n')))
+						if( ( data = strchr( proto , ' ')))
 						{
-							*p = '\0' ;
+							*data = '\0' ;
+							data += 2 ;
+
+							if( ( p = strchr( data , '\n')))
+							{
+								*p = '\0' ;
+							}
 						}
 					}
 				}
+
+				pclose( fp) ;
 			}
 
-			fclose( fp) ;
+	#ifndef  TRUSTED
+			unlink( xauth_file) ;
 		}
+
+		free( xauth_file) ;
+	#endif
 	}
+#endif
 
 #ifdef  __DEBUG
 	kik_debug_printf( KIK_DEBUG_TAG " libssh2_channel_x11_req_ex (with xauth %s %s)\n" ,
@@ -1187,7 +1314,7 @@ setup_x11(
 
 static void
 x11_callback(
-	LIBSSH2_SESSION *  session ,
+	LIBSSH2_SESSION *  session_obj ,
 	LIBSSH2_CHANNEL *  channel ,
 	char *  shost ,
 	int  sport ,
@@ -1195,74 +1322,128 @@ x11_callback(
 	)
 {
 	u_int  count ;
-	int  channel_sock ;
+	ssh_session_t *  session ;
+	void *  p ;
+	int  display_sock = -1 ;
+#ifdef  USE_WIN32API
+	struct sockaddr_in  addr ;
+#else
 	struct sockaddr_un  addr ;
-	int  display_sock ;
-
-	if( display_port == -1 || num_of_x11_fds >= 10)
-	{
-		goto  error ;
-	}
+#endif
 
 	for( count = 0 ; ; count++)
 	{
 		if( count == num_of_sessions)
 		{
-			goto  error ;
+			/* XXX count must not reache num_of_sessions. */
+			return ;
 		}
 
-		if( session == sessions[count]->obj)
+		if( session_obj == sessions[count]->obj)
 		{
-			channel_sock = sessions[count]->sock ;
+			session = sessions[count] ;
 			break ;
 		}
 	}
 
+	if( ! ( p = realloc( session->x11_fds ,
+			(session->num_of_x11 + 1) * sizeof(int))))
+	{
+		/* XXX channel resource is leaked. */
+		return ;
+	}
+
+	session->x11_fds = p ;
+
+	if( ! ( p = realloc( session->x11_channels ,
+			(session->num_of_x11 + 1) * sizeof(LIBSSH2_CHANNEL*))))
+	{
+		/* XXX channel resource is leaked. */
+		return ;
+	}
+
+	session->x11_channels = p ;
+
+	if( display_port == -1)
+	{
+		goto  error ;
+	}
+
+#ifdef  USE_WIN32API
+	if( ( display_sock = socket( AF_INET , SOCK_STREAM , 0)) < 0)
+	{
+		goto  error ;
+	}
+
+	memset( &addr , 0 , sizeof(addr)) ;
+	addr.sin_family = AF_INET ;
+	addr.sin_port = htons(6000) ;
+	addr.sin_addr.s_addr = htonl(INADDR_LOOPBACK) ;
+#else
 	if( ( display_sock = socket( AF_UNIX , SOCK_STREAM , 0)) < 0)
 	{
 		goto  error ;
 	}
 
 	memset( &addr , 0 , sizeof(addr)) ;
-	addr.sun_family = AF_UNIX;
+	addr.sun_family = AF_UNIX ;
 	snprintf( addr.sun_path , sizeof(addr.sun_path) ,
 		"/tmp/.X11-unix/X%d" , display_port) ;
+#endif
 
-	if( connect( display_sock , (struct sockaddr *)&addr , sizeof(addr)) != -1)
+	if( connect( display_sock , (struct sockaddr *)&addr , sizeof(addr)) < 0)
+	{
+	error:
+		kik_msg_printf( "Failed to connect X Server.\n") ;
+		closesocket( display_sock) ;
+		display_sock = -1 ;
+
+		/* Don't call libssh2_channel_free() which causes segfault here. */
+	}
+	else
 	{
 	#ifdef  USE_WIN32API
 		u_long  val ;
 
+		val = 1 ;
 		ioctlsocket( display_sock , FIONBIO , &val) ;
 	#else
 		fcntl( display_sock , F_SETFL , O_NONBLOCK|fcntl( display_sock , F_GETFL , 0)) ;
 	#endif
-
-		sessions[count]->ref_count ++ ;
-
-		x11_channels[num_of_x11_fds].channel = channel ;
-		x11_channels[num_of_x11_fds].session = sessions[count] ;
-		x11_fds[num_of_x11_fds].display = display_sock ;
-		x11_fds[num_of_x11_fds++].channel = channel_sock ;
-
-	#ifdef  __DEBUG
-		kik_debug_printf( KIK_DEBUG_TAG " x11 forwarding started.\n") ;
-	#endif
-
-		/* Successfully setup x11 forwarding. */
-
-		return ;
-	}
-	else
-	{
-		closesocket( display_sock) ;
 	}
 
-error:
-	libssh2_channel_free( channel) ;
+	session->x11_channels[session->num_of_x11] = channel ;
+	session->x11_fds[session->num_of_x11++] = display_sock ;
+
+#ifdef  __DEBUG
+	kik_debug_printf( KIK_DEBUG_TAG
+		" x11 forwarding %d (display %d <=> ssh %p) started. => channel num %d\n" ,
+		session->num_of_x11 - 1 , display_sock , channel , session->num_of_x11) ;
+#endif
 }
 
-static int  ssh_to_xserver( LIBSSH2_CHANNEL *  channel , int  display) ;
+static void
+close_x11(
+	ssh_session_t *  session ,
+	int  idx
+	)
+{
+	closesocket( session->x11_fds[idx]) ;
+	while( libssh2_channel_free( session->x11_channels[idx]) == LIBSSH2_ERROR_EAGAIN) ;
+
+#ifdef  __DEBUG
+	kik_debug_printf( KIK_DEBUG_TAG
+		" x11 forwarding %d (display %d <=> ssh %p) stopped. => channel num %d\n" ,
+		idx , session->x11_fds[idx] , session->x11_channels[idx] ,
+		session->num_of_x11 - 1) ;
+#endif
+
+	if( -- session->num_of_x11 > 0)
+	{
+		session->x11_channels[idx] = session->x11_channels[session->num_of_x11] ;
+		session->x11_fds[idx] = session->x11_fds[session->num_of_x11] ;
+	}
+}
 
 static int
 xserver_to_ssh(
@@ -1272,10 +1453,8 @@ xserver_to_ssh(
 {
 	char  buf[8192] ;
 	ssize_t  len ;
-	int  ret ;
 
-	ret = -1 ;
-	while( ( len = read( display , buf , sizeof(buf))) > 0)
+	while( ( len = recv( display , buf , sizeof(buf) , 0)) > 0)
 	{
 		ssize_t  w_len ;
 		char *  p ;
@@ -1288,13 +1467,23 @@ xserver_to_ssh(
 				len -= w_len ;
 				p += w_len ;
 			}
+			else if( w_len < 0)
+			{
+				if( libssh2_channel_eof( channel))
+				{
+				#if  0
+					shutdown( display , SHUT_RDWR) ;
+				#endif
 
-			ssh_to_xserver( channel , display) ;
+					return  0 ;
+				}
+
+				kik_usleep(1) ;
+			}
 		}
 
-		ret = 1 ;
 	#if  0
-		kik_debug_printf( "X SERVER -> CHANNEL %d\n" , len) ;
+		kik_debug_printf( "X SERVER(%d) -> CHANNEL %d\n" , display , len) ;
 	#endif
 	}
 
@@ -1304,7 +1493,7 @@ xserver_to_ssh(
 	}
 	else
 	{
-		return  ret ;
+		return  1 ;
 	}
 }
 
@@ -1316,45 +1505,45 @@ ssh_to_xserver(
 {
 	char  buf[8192] ;
 	ssize_t  len ;
-	int  ret ;
 
-	ret = -1 ;
 	while( ( len = libssh2_channel_read( channel , buf , sizeof(buf))) > 0)
 	{
 		ssize_t  w_len ;
 		char *  p ;
 
 		p = buf ;
-		while( ( w_len = write( display , p , len)) < len)
+		while( ( w_len = send( display , p , len , 0)) < len)
 		{
 			if( w_len > 0)
 			{
 				len -= w_len ;
 				p += w_len ;
 			}
-
-			xserver_to_ssh( channel , display) ;
+			else if( w_len < 0)
+			{
+				kik_usleep(1) ;
+			}
 		}
 
-		ret = 1 ;
-
 	#if  0
-		kik_debug_printf( "CHANNEL -> X SERVER %d\n" , len) ;
+		kik_debug_printf( "CHANNEL -> X SERVER(%d) %d\n" , display , len) ;
 	#endif
 	}
 
 	if( libssh2_channel_eof( channel))
 	{
+	#if  0
+		shutdown( display , SHUT_RDWR) ;
+	#endif
+
 		return  0 ;
 	}
 	else
 	{
-		return  ret ;
+		return  1 ;
 	}
 }
 
-#endif	/* USE_WIN32API */
-
 
 /* --- global functions --- */
 
@@ -1377,6 +1566,7 @@ ml_pty_ssh_new(
 	char *  host ;
 	char *  port ;
 	char *  term ;
+	void *  p ;
 
 	if( ( pty = calloc( 1 , sizeof( ml_pty_ssh_t))) == NULL)
 	{
@@ -1406,15 +1596,14 @@ ml_pty_ssh_new(
 		goto  error1 ;
 	}
 
-#ifndef  USE_WIN32API
-	if( use_x11_forwarding)
+	if( ! ( p = realloc( pty->session->pty_channels ,
+			(pty->session->num_of_ptys + 1) * sizeof(LIBSSH2_CHANNEL*))))
 	{
-		libssh2_session_callback_set( pty->session->obj ,
-			LIBSSH2_CALLBACK_X11 , x11_callback) ;
+		goto  error1 ;
 	}
-#endif
 
-	/* Request a shell */
+	pty->session->pty_channels = p ;
+
 	if( ! ( pty->channel = libssh2_channel_open_session( pty->session->obj)))
 	{
 	#ifdef  DEBUG
@@ -1426,6 +1615,18 @@ ml_pty_ssh_new(
 
 	pty->session->doing_scp = 0 ;
 
+#ifdef  LIBSSH2_FORWARD_AGENT
+	if( auth_agent_is_available)
+	{
+		if( libssh2_channel_request_auth_agent( pty->channel) == 0)
+		{
+			kik_msg_printf( "Agent forwarding.\n") ;
+		}
+
+		auth_agent_is_available = 0 ;
+	}
+#endif
+
 	term = NULL ;
 	if( env)
 	{
@@ -1465,23 +1666,19 @@ ml_pty_ssh_new(
 	if( libssh2_channel_request_pty( pty->channel , term ? term : "xterm"))
 	{
 	#ifdef  DEBUG
-		kik_debug_printf( KIK_DEBUG_TAG " Failed requesting pty\n") ;
+		kik_debug_printf( KIK_DEBUG_TAG " Failed to request pty\n") ;
 	#endif
 
 		goto  error3 ;
 	}
 
-#ifndef  USE_WIN32API
-	if( use_x11_forwarding)
+	if( pty->session->use_x11_forwarding)
 	{
 		if( ! setup_x11( pty->channel))
 		{
-		#ifdef  DEBUG
-			kik_debug_printf( KIK_DEBUG_TAG " Failed requesting x11\n") ;
-		#endif
+			kik_msg_printf( "X11 forwarding failed.\n") ;
 		}
 	}
-#endif
 
 	if( cmd_path)
 	{
@@ -1569,7 +1766,8 @@ ml_pty_ssh_new(
 		if( GetLastError() != ERROR_ALREADY_EXISTS)
 		{
 			/* Launch the thread that wait for receiving data from pty. */
-			if( ! ( thrd = _beginthreadex( NULL , 0 , wait_pty_read , pty , 0 , &tid)))
+			if( ! ( thrd = _beginthreadex( NULL , 0 , wait_pty_read ,
+						NULL , 0 , &tid)))
 			{
 			#ifdef  DEBUG
 				kik_warn_printf( KIK_DEBUG_TAG " CreateThread() failed.\n") ;
@@ -1584,9 +1782,13 @@ ml_pty_ssh_new(
 		{
 			/* java/MLTerm.java has already watched pty. */
 		}
+
+		main_tid = GetCurrentThreadId() ;
 	}
 #endif
 
+	pty->session->pty_channels[pty->session->num_of_ptys++] = pty->channel ;
+
 	return  &pty->pty ;
 
 error3:
@@ -1814,10 +2016,61 @@ ml_pty_ssh_keepalive(
 
 void
 ml_pty_ssh_set_use_x11_forwarding(
+	void *  session ,
 	int  use
 	)
 {
-	use_x11_forwarding = use ;
+	if( session)
+	{
+		((ssh_session_t*)session)->use_x11_forwarding = use ;
+	}
+	else
+	{
+		use_x11_forwarding = use ;
+	}
+}
+
+int
+ml_pty_ssh_poll(
+	void *  p
+	)
+{
+	fd_set *  fds ;
+	int  num_of_fds ;
+	u_int  count ;
+
+	fds = p ;
+	FD_ZERO(fds) ;
+	num_of_fds = 0 ;
+
+	for( count = 0 ; count < num_of_sessions ; count++)
+	{
+		u_int  idx ;
+
+		for( idx = 0 ; idx < sessions[count]->num_of_ptys ; idx++)
+		{
+			if( libssh2_poll_channel_read( sessions[count]->pty_channels[idx] , 0))
+			{
+				goto  found ;
+			}
+		}
+
+		for( idx = 0 ; idx < sessions[count]->num_of_x11 ; idx++)
+		{
+			if( libssh2_poll_channel_read( sessions[count]->x11_channels[idx] , 0))
+			{
+				goto  found ;
+			}
+		}
+
+		continue ;
+
+	found:
+		FD_SET( sessions[count]->sock , fds) ;
+		num_of_fds ++ ;
+	}
+
+	return  num_of_fds ;
 }
 
 u_int
@@ -1825,87 +2078,82 @@ ml_pty_ssh_get_x11_fds(
 	int **  fds
 	)
 {
-#ifndef  USE_WIN32API
-	if( num_of_x11_fds > 0)
-	{
-		int  count ;
+	static int *  x11_fds ;
+	static u_int  num_of_x11_fds ;
+	u_int  count ;
+	u_int  num ;
 
-		for( count = num_of_x11_fds - 1 ; count >= 0 ; count--)
-		{
-			if( ! x11_channels[count].channel)
-			{
-				/* Already closed in ml_pty_ssh_send_recv_x11(). */
-				x11_fds[count] = x11_fds[--num_of_x11_fds] ;
-				x11_channels[count] = x11_channels[num_of_x11_fds] ;
-			}
-		#if  1
-			else
-			{
-				/*
-				 * Packets from ssh channels aren't necessarily
-				 * detected by select(), so are received here
-				 * in a certain interval.
-				 * ( "+ 1" means x11_fd_t.channel.)
-				 */
-				ml_pty_ssh_send_recv_x11( count * 2 + 1) ;
-			}
-		#endif
-		}
-
-		*fds = x11_fds ;
-
-		return  num_of_x11_fds * 2 ;
-	}
-	else
-#endif
+	if( num_of_sessions == 0)
 	{
 		return  0 ;
 	}
+
+	num = 0 ;
+	for( count = 0 ; count < num_of_sessions ; count++)
+	{
+		num += sessions[count]->num_of_x11 ;
+	}
+
+	if( num_of_x11_fds < num)
+	{
+		void *  p ;
+
+		num_of_x11_fds = num ;
+
+		if( ! ( p = realloc( x11_fds , num * sizeof(int))))
+		{
+			return  0 ;
+		}
+
+		x11_fds = p ;
+	}
+
+	num = 0 ;
+	for( count = 0 ; count < num_of_sessions ; count++)
+	{
+		memcpy( x11_fds + num , sessions[count]->x11_fds ,
+			sessions[count]->num_of_x11 * sizeof(int)) ;
+		num += sessions[count]->num_of_x11 ;
+	}
+
+	*fds = x11_fds ;
+
+	return  num ;
 }
 
 int
 ml_pty_ssh_send_recv_x11(
-	int  idx
+	int  idx ,
+	int  bidirection
 	)
 {
-#ifndef  USE_WIN32API
-	int  read_channel ;
-	int  ret ;
+	u_int  count ;
+	ssh_session_t *  session ;
 
-	if( idx >= num_of_x11_fds * 2)
+	for( count = 0 ; ; count++)
 	{
-		return  0 ;
+		if( count >= num_of_sessions)
+		{
+			return  0 ;
+		}
+
+		if( idx < sessions[count]->num_of_x11)
+		{
+			break ;
+		}
+
+		idx -= sessions[count]->num_of_x11 ;
 	}
 
-	read_channel = idx % 2 ;
-	idx /= 2 ;
+	session = sessions[count] ;
 
-	if( read_channel)
+	if( session->x11_fds[idx] == -1 ||	/* Failed to connect X server */
+	    ! ( ( ! bidirection ||
+	          xserver_to_ssh( session->x11_channels[idx] , session->x11_fds[idx])) &&
+	        ssh_to_xserver( session->x11_channels[idx] , session->x11_fds[idx])))
 	{
-		ret = ssh_to_xserver( x11_channels[idx].channel , x11_fds[idx].display) ;
-	}
-	else
-	{
-		ret = xserver_to_ssh( x11_channels[idx].channel , x11_fds[idx].display) ;
-	}
-
-	if( ! ret)
-	{
-		closesocket( x11_fds[idx].display) ;
-
-		libssh2_session_set_blocking( x11_channels[idx].session->obj , 1) ;
-		libssh2_channel_free( x11_channels[idx].channel) ;
-		ssh_disconnect( x11_channels[idx].session) ;
-
-		x11_channels[idx].channel = NULL ;
-
-	#ifdef  __DEBUG
-		kik_debug_printf( KIK_DEBUG_TAG " x11 forwarding finished.\n") ;
-	#endif
+		close_x11( session , idx) ;
 	}
 
 	return  1 ;
-#else
-	return  0 ;
-#endif
 }
