@@ -9,15 +9,15 @@
 #include  <kiklib/kik_util.h>	/* K_MIN */
 
 
-#define  CHARSET(attr)  (((attr) >> 7) & 0x1ff)
+#define  MLFONT(attr)  (((attr) >> 5) & 0x7fff)
+#define  IS_ITALIC(attr)  ((attr) & (0x1 << 16))
+#define  IS_BOLD(attr)  ((attr) & (0x1 << 15))
+#define  IS_FULLWIDTH(attr)  ((attr) & (0x1 << 14))
+#define  CHARSET(attr)  (((attr) >> 5) & 0x1ff)
 
-#define  IS_BIWIDTH(attr)  ((attr) & (0x1 << 6))
-
-#define  IS_REVERSED(attr)  ((attr) & (0x1 << 5))
-#define  REVERSE_COLOR(attr) ((attr) |= (0x1 << 5))
-#define  RESTORE_COLOR(attr) ((attr) &= ~(0x1 << 5))
-
-#define  IS_BOLD(attr)  ((attr) & (0x1 << 4))
+#define  IS_REVERSED(attr)  ((attr) & (0x1 << 4))
+#define  REVERSE_COLOR(attr) ((attr) |= (0x1 << 4))
+#define  RESTORE_COLOR(attr) ((attr) &= ~(0x1 << 4))
 
 #define  IS_UNDERLINED(attr)  ((attr) & (0x1 << 3))
 
@@ -25,16 +25,16 @@
 
 #define  IS_COMB_TRAILING(attr)  ((attr) & (0x1 << 1))
 #define  SET_COMB_TRAILING(attr)  ((attr) |= (0x1 << 1))
-#define  UNSET_COMB_TRAILING(attr)  ((attr) &= 0xfffd)
+#define  UNSET_COMB_TRAILING(attr)  ((attr) &= 0xfffffd)
 
 #define  IS_SINGLE_CH(attr)  ((attr) & 0x1)
-#define  USE_MULTI_CH(attr)  ((attr) &= 0xfffe)
+#define  USE_MULTI_CH(attr)  ((attr) &= 0xfffffe)
 #define  UNUSE_MULTI_CH(attr)  ((attr) |= 0x1)
 
-#define  COMPOUND_ATTR(charset,is_biwidth,is_bold,is_underlined,is_comb) \
-	( ((charset) << 7) | ((is_biwidth) << 6) | \
-	( 0x0 << 5) | ((is_bold) << 4) | ((is_underlined) << 3) | \
-	( (is_comb) << 2) | ( 0x0 << 1) | 0x1)
+#define  COMPOUND_ATTR(charset,is_fullwidth,is_bold,is_italic,unicode_area,is_underlined,is_comb) \
+	( ((unicode_area) << 17) | ((is_italic) << 16) | ((is_bold) << 15) | \
+	  ((is_fullwidth) << 14) | ((charset) << 5) | ( 0x0 << 4) |  \
+	  ((is_underlined) << 3) | ((is_comb) << 2) | ( 0x0 << 1) | 0x1)
 
 
 /* --- static variables --- */
@@ -42,6 +42,14 @@
 static int  use_char_combining = 1 ;
 static int  use_multi_col_char = 1 ;
 
+static struct
+{
+	u_int32_t  min ;
+	u_int32_t  max ;
+
+} *  unicode_areas ;
+static u_int  num_of_unicode_areas ;
+
 
 /* --- static functions --- */
 
@@ -62,56 +70,6 @@ get_comb_size(
 	return  size ;
 }
 
-/* See also ml_get_closest_color() in ml_color.c. */
-inline static u_int8_t
-intern_color(
-	ml_color_t  ex_color
-	)
-{
-	if( ex_color == ML_FG_COLOR)
-	{
-		return  0x10 ;
-	}
-	else if( ex_color == ML_BG_COLOR)
-	{
-		return  0xe7 ;
-	}
-	else if( IS_256_COLOR(ex_color))
-	{
-		if( ex_color == 0x10)
-		{
-			/* avoid batting internal presentation of ML_FG_COLOR. */
-			return  ML_BLACK ;
-		}
-		else if( ex_color == 0xe7)
-		{
-			/* avoid batting internal presentation of ML_BG_COLOR. */
-			return  ML_WHITE | ML_BOLD_COLOR_MASK ;
-		}
-	}
-	
-	return  ex_color ;
-}
-
-inline static ml_color_t
-extern_color(
-	u_int8_t  in_color
-	)
-{
-	if( in_color == 0x10)
-	{
-		return  ML_FG_COLOR ;
-	}
-	else if( in_color == 0xe7)
-	{
-		return  ML_BG_COLOR ;
-	}
-	else
-	{
-		return  in_color ;
-	}
-}
-
 
 /* --- global functions --- */
 
@@ -135,6 +93,28 @@ ml_set_use_multi_col_char(
 	return  1 ;
 }
 
+ml_font_t
+ml_char_add_unicode_area_font(
+	u_int32_t  min ,
+	u_int32_t  max
+	)
+{
+	void *  p ;
+
+	if( num_of_unicode_areas >= 7 ||
+	    ! ( p = realloc( unicode_areas ,
+			sizeof(*unicode_areas) * (num_of_unicode_areas + 1))))
+	{
+		return  UNKNOWN_CS ;
+	}
+
+	unicode_areas = p ;
+	unicode_areas[num_of_unicode_areas].min = min ;
+	unicode_areas[num_of_unicode_areas++].max = max ;
+
+	return  ISO10646_UCS4_1 | (num_of_unicode_areas << 12) ;
+}
+
 
 /*
  * character functions
@@ -178,40 +158,21 @@ ml_char_final(
 int
 ml_char_set(
 	ml_char_t *  ch ,
-	u_char *  bytes ,
-	size_t  size ,
+	u_int32_t  code ,
 	mkf_charset_t  cs ,
-	int  is_biwidth ,
+	int  is_fullwidth ,
 	int  is_comb ,
 	ml_color_t  fg_color ,
 	ml_color_t  bg_color ,
 	int  is_bold ,
+	int  is_italic ,
 	int  is_underlined
 	)
 {
-#ifdef  DEBUG
-	/* Ignore PICTURE_CHARSET. */
-	if( cs != PICTURE_CHARSET && CS_SIZE(cs) != size)
-	{
-		kik_debug_printf( KIK_DEBUG_TAG " char size error => size %d <=> CS_SIZE %d\n",
-			size, CS_SIZE(cs)) ;
-
-		return  0 ;
-	}
-
-	if( size > MLCHAR_SIZE)
-	{
-		kik_debug_printf( KIK_DEBUG_TAG " size %d is over MAX CHAR SIZE %d\n",
-			size, MLCHAR_SIZE) ;
-
-		return  0 ;
-	}
-#endif
-
+	u_int  idx ;
 	ml_char_final( ch) ;
 
-	memcpy( ch->u.ch.bytes , bytes , size) ;
-	memset( ch->u.ch.bytes + size , 0 , MLCHAR_SIZE - size) ;
+	ch->u.ch.code = code ;
 
 #ifdef  DEBUG
 	if( cs >= MAX_CHARSET)
@@ -220,9 +181,26 @@ ml_char_set(
 	}
 #endif
 
-	ch->u.ch.attr = COMPOUND_ATTR(cs,is_biwidth!=0,is_bold!=0,is_underlined!=0,is_comb!=0) ;
-	ch->u.ch.fg_color = intern_color(fg_color) ;
-	ch->u.ch.bg_color = intern_color(bg_color) ;
+	if( unicode_areas && cs == ISO10646_UCS4_1)
+	{
+		for( idx = num_of_unicode_areas ; idx > 0 ; idx --)
+		{
+			if( unicode_areas[idx - 1].min <= code &&
+			    code <= unicode_areas[idx - 1].max)
+			{
+				break ;
+			}
+		}
+	}
+	else
+	{
+		idx = 0 ;
+	}
+
+	ch->u.ch.attr = COMPOUND_ATTR(cs,is_fullwidth!=0,is_bold!=0,is_italic!=0,idx,
+				is_underlined!=0,is_comb!=0) ;
+	ch->u.ch.fg_color = fg_color ;
+	ch->u.ch.bg_color = bg_color ;
 
 	return  1 ;
 }
@@ -230,14 +208,14 @@ ml_char_set(
 int
 ml_char_combine(
 	ml_char_t *  ch ,
-	u_char *  bytes ,
-	size_t  size ,
+	u_int32_t  code ,
 	mkf_charset_t  cs ,
-	int  is_biwidth ,
+	int  is_fullwidth ,
 	int  is_comb ,
 	ml_color_t  fg_color ,
 	ml_color_t  bg_color ,
 	int  is_bold ,
+	int  is_italic ,
 	int  is_underlined
 	)
 {
@@ -293,8 +271,8 @@ ml_char_combine(
 		SET_COMB_TRAILING( multi_ch->u.ch.attr) ;
 
 		ml_char_init( multi_ch + 1) ;
-		if( ml_char_set( multi_ch + 1 , bytes , size , cs , is_biwidth , is_comb ,
-			fg_color , bg_color , is_bold , is_underlined) == 0)
+		if( ml_char_set( multi_ch + 1 , code , cs , is_fullwidth , is_comb ,
+			fg_color , bg_color , is_bold , is_italic , is_underlined) == 0)
 		{
 			return  0 ;
 		}
@@ -332,15 +310,15 @@ ml_char_combine(
 
 		SET_COMB_TRAILING( multi_ch[comb_size].u.ch.attr) ;
 		ml_char_init( multi_ch + comb_size + 1) ;
-		if( ml_char_set( multi_ch + comb_size + 1 , bytes , size , cs , is_biwidth ,
-			is_comb , fg_color , bg_color , is_bold , is_underlined) == 0)
+		if( ml_char_set( multi_ch + comb_size + 1 , code , cs , is_fullwidth ,
+			is_comb , fg_color , bg_color , is_bold , is_italic , is_underlined) == 0)
 		{
 			return  0 ;
 		}
 	}
 
 	ch->u.multi_ch = multi_ch ;
-	USE_MULTI_CH(ch->u.ch.attr) ;
+	USE_MULTI_CH(ch->u.ch.attr) ;	/* necessary for 64bit big endian */
 
 	return  1 ;
 }
@@ -351,13 +329,11 @@ ml_char_combine_simple(
 	ml_char_t *  comb
 	)
 {
-	mkf_charset_t  cs ;
-
-	cs = CHARSET(comb->u.ch.attr) ;
-	return  ml_char_combine( ch , ml_char_bytes( comb) , CS_SIZE(cs) , cs ,
-			IS_BIWIDTH(comb->u.ch.attr) , IS_COMB(comb->u.ch.attr) ,
+	return  ml_char_combine( ch , ml_char_code( comb) , CHARSET(comb->u.ch.attr) ,
+			IS_FULLWIDTH(comb->u.ch.attr) , IS_COMB(comb->u.ch.attr) ,
 			comb->u.ch.fg_color , comb->u.ch.bg_color ,
-			IS_BOLD(comb->u.ch.attr) , IS_UNDERLINED(comb->u.ch.attr)) ;
+			IS_BOLD(comb->u.ch.attr) , IS_ITALIC(comb->u.ch.attr) ,
+			IS_UNDERLINED(comb->u.ch.attr)) ;
 }
 
 ml_char_t *
@@ -457,63 +433,40 @@ ml_char_copy(
 		memcpy( multi_ch , src->u.multi_ch , sizeof( ml_char_t) * (comb_size + 1)) ;
 
 		dst->u.multi_ch = multi_ch ;
-		USE_MULTI_CH(dst->u.ch.attr) ;
+		USE_MULTI_CH(dst->u.ch.attr) ;	/* necessary for 64bit big endian */
 	}
 	
 	return  1 ;
 }
 
-u_char *
-ml_char_bytes(
+u_int32_t
+ml_char_code(
 	ml_char_t *  ch
 	)
 {
 	if( IS_SINGLE_CH(ch->u.ch.attr))
 	{
-		return  ch->u.ch.bytes ;
+		return  ch->u.ch.code ;
 	}
 	else
 	{
-		return  ml_char_bytes( ch->u.multi_ch) ;
-	}
-}
-
-size_t
-ml_char_size(
-	ml_char_t *  ch
-	)
-{
-	if( IS_SINGLE_CH(ch->u.ch.attr))
-	{
-		mkf_charset_t  cs ;
-
-		cs = CHARSET(ch->u.ch.attr) ;
-
-		return  CS_SIZE(cs) ;
-	}
-	else
-	{
-		return  ml_char_size( ch->u.multi_ch) ;
+		return  ml_char_code( ch->u.multi_ch) ;
 	}
 }
 
 int
-ml_char_set_bytes(
+ml_char_set_code(
 	ml_char_t *  ch ,
-	u_char *  bytes
+	u_int32_t  code
 	)
 {
 	if( IS_SINGLE_CH(ch->u.ch.attr))
 	{
-		mkf_charset_t  cs ;
-
-		cs = CHARSET(ch->u.ch.attr) ;
-
-		memcpy( ch->u.ch.bytes , bytes , CS_SIZE(cs)) ;
+		ch->u.ch.code = code ;
 	}
 	else
 	{
-		ml_char_set_bytes( ch->u.multi_ch , bytes) ;
+		ml_char_set_code( ch->u.multi_ch , code) ;
 	}
 	
 	return  1 ;
@@ -545,9 +498,7 @@ ml_char_font(
 
 	if( IS_SINGLE_CH(attr))
 	{
-		return  CHARSET(attr) |
-		        (IS_BOLD(attr) ? FONT_BOLD : 0) |
-		        (IS_BIWIDTH(attr) ? FONT_BIWIDTH : 0) ;
+		return  MLFONT(attr) ;
 	}
 	else
 	{
@@ -564,58 +515,67 @@ ml_char_cols(
 	ml_char_t *  ch
 	)
 {
-	if( use_multi_col_char && ml_char_is_biwidth(ch))
-	{
-		return  2 ;
-	}
-#if  1
-	else if( IS_SINGLE_CH(ch->u.ch.attr) && CHARSET(ch->u.ch.attr) == ISO10646_UCS4_1)
-	{
-		/*
-		 * 0 should be returned for all zero-width characters of Unicode,
-		 * but 0 is returned for following characters alone for now.
-		 * 200C;ZERO WIDTH NON-JOINER
-		 * 200D;ZERO WIDTH JOINER
-		 * 200E;LEFT-TO-RIGHT MARK
-		 * 200F;RIGHT-TO-LEFT MARK
-		 * 202A;LEFT-TO-RIGHT EMBEDDING
-		 * 202B;RIGHT-TO-LEFT EMBEDDING
-		 * 202C;POP DIRECTIONAL FORMATTING
-		 * 202D;LEFT-TO-RIGHT OVERRIDE
-		 * 202E;RIGHT-TO-LEFT OVERRIDE
-		*/
-		u_char *  bytes ;
+	u_int  attr ;
 
-		bytes = ch->u.ch.bytes ;
+	attr = ch->u.ch.attr ;
 
-		if( bytes[2] == 0x20 &&
-		    ((0x0c <= bytes[3] && bytes[3] <= 0x0f) ||
-		     (0x2a <= bytes[3] && bytes[3] <= 0x2e)) &&
-		    bytes[0] == 0 && bytes[1] == 0)
+	if( IS_SINGLE_CH(attr))
+	{
+		if( use_multi_col_char && IS_FULLWIDTH(attr))
 		{
-			return  0 ;
+			return  2 ;
 		}
-	}
-#endif
+	#if  1
+		else if( CHARSET(attr) == ISO10646_UCS4_1)
+		{
+			/*
+			 * 0 should be returned for all zero-width characters of Unicode,
+			 * but 0 is returned for following characters alone for now.
+			 * 200C;ZERO WIDTH NON-JOINER
+			 * 200D;ZERO WIDTH JOINER
+			 * 200E;LEFT-TO-RIGHT MARK
+			 * 200F;RIGHT-TO-LEFT MARK
+			 * 202A;LEFT-TO-RIGHT EMBEDDING
+			 * 202B;RIGHT-TO-LEFT EMBEDDING
+			 * 202C;POP DIRECTIONAL FORMATTING
+			 * 202D;LEFT-TO-RIGHT OVERRIDE
+			 * 202E;RIGHT-TO-LEFT OVERRIDE
+			*/
+			u_int32_t  code ;
 
-	return  1 ;
+			code = ch->u.ch.code ;
+
+			if( ( 0x200c <= code && code <= 0x200f) ||
+			    ( 0x202a <= code && code <= 0x202e))
+			{
+				return  0 ;
+			}
+		}
+	#endif
+
+		return  1 ;
+	}
+	else
+	{
+		return  ml_char_cols( ch->u.multi_ch) ;
+	}
 }
 
 /*
  * 'use_multi_col_char' not concerned.
  */
 u_int
-ml_char_is_biwidth(
+ml_char_is_fullwidth(
 	ml_char_t *  ch
 	)
 {
 	if( IS_SINGLE_CH(ch->u.ch.attr))
 	{
-		return  IS_BIWIDTH(ch->u.ch.attr) ;
+		return  IS_FULLWIDTH(ch->u.ch.attr) ;
 	}
 	else
 	{
-		return  ml_char_is_biwidth( ch->u.multi_ch) ;
+		return  ml_char_is_fullwidth( ch->u.multi_ch) ;
 	}
 }
 
@@ -649,7 +609,7 @@ ml_char_fg_color(
 
 		if( IS_REVERSED(attr))
 		{
-			color = extern_color( ch->u.ch.bg_color) ;
+			color = ch->u.ch.bg_color ;
 
 			/*
 			 * xterm doesn't use hilighted color for bold background color.
@@ -657,7 +617,7 @@ ml_char_fg_color(
 		}
 		else
 		{
-			color = extern_color( ch->u.ch.fg_color) ;
+			color = ch->u.ch.fg_color ;
 
 			if( color < MAX_VTSYS_COLORS && IS_BOLD(attr))
 			{
@@ -681,9 +641,7 @@ ml_char_set_fg_color(
 {
 	if( IS_SINGLE_CH(ch->u.ch.attr))
 	{
-		ch->u.ch.fg_color = intern_color(color) ;
-
-		return  1 ;
+		ch->u.ch.fg_color = color ;
 	}
 	else
 	{
@@ -695,9 +653,9 @@ ml_char_set_fg_color(
 		{
 			ml_char_set_fg_color( ch->u.multi_ch + count , color) ;
 		}
-
-		return  1 ;
 	}
+
+	return  1 ;
 }
 
 ml_color_t
@@ -715,7 +673,7 @@ ml_char_bg_color(
 
 		if( IS_REVERSED(attr))
 		{
-			color = extern_color(ch->u.ch.fg_color) ;
+			color = ch->u.ch.fg_color ;
 
 			if( color < MAX_VTSYS_COLORS && IS_BOLD(attr))
 			{
@@ -724,7 +682,7 @@ ml_char_bg_color(
 		}
 		else
 		{
-			color = extern_color(ch->u.ch.bg_color) ;
+			color = ch->u.ch.bg_color ;
 
 			/*
 			 * xterm doesn't use hilighted color for bold background color.
@@ -747,9 +705,7 @@ ml_char_set_bg_color(
 {
 	if( IS_SINGLE_CH(ch->u.ch.attr))
 	{
-		ch->u.ch.bg_color = intern_color(color) ;
-
-		return  1 ;
+		ch->u.ch.bg_color = color ;
 	}
 	else
 	{
@@ -761,9 +717,9 @@ ml_char_set_bg_color(
 		{
 			ml_char_set_bg_color( ch->u.multi_ch + count , color) ;
 		}
-
-		return  1 ;
 	}
+
+	return  1 ;
 }
 
 int
@@ -882,11 +838,7 @@ ml_char_is_null(
 {
 	if( IS_SINGLE_CH( ch->u.ch.attr))
 	{
-		mkf_charset_t  cs ;
-
-		cs = CHARSET(ch->u.ch.attr) ;
-
-		return  (CS_SIZE(cs) == 1 && ch->u.ch.bytes[0] == '\0') ;
+		return  ch->u.ch.code == 0 ;
 	}
 	else
 	{
@@ -897,7 +849,7 @@ ml_char_is_null(
 /*
  * XXX
  * Returns inaccurate result in dealing with combined characters.
- * Even if they have the same bytes, false is returned since
+ * Even if they have the same code, false is returned since
  * ml_char_t:multi_ch-s never point the same address.)
  */
 int
@@ -910,18 +862,15 @@ ml_char_equal(
 }
 
 int
-ml_char_bytes_is(
+ml_char_code_is(
 	ml_char_t *  ch ,
-	char *  bytes ,
-	size_t  size ,
+	u_int32_t  code ,
 	mkf_charset_t  cs
 	)
 {
 	if( IS_SINGLE_CH(ch->u.ch.attr))
 	{
-		if( CHARSET(ch->u.ch.attr) == cs &&
-		    CS_SIZE(cs) == size &&
-		    memcmp( ml_char_bytes( ch) , bytes , size) == 0)
+		if( CHARSET(ch->u.ch.attr) == cs && ch->u.ch.code == code)
 		{
 			return  1 ;
 		}
@@ -932,37 +881,27 @@ ml_char_bytes_is(
 	}
 	else
 	{
-		return  ml_char_bytes_is( ch->u.multi_ch , bytes , size , cs) ;
+		return  ml_char_code_is( ch->u.multi_ch , code , cs) ;
 	}
 }
 
 int
-ml_char_bytes_equal(
+ml_char_code_equal(
 	ml_char_t *  ch1 ,
 	ml_char_t *  ch2
 	)
 {
-	u_int  size1 ;
-	u_int  size2 ;
 	ml_char_t *  comb1 ;
 	ml_char_t *  comb2 ;
 	u_int  comb1_size ;
 	u_int  comb2_size ;
 	u_int  count ;
 
-	size1 = ml_char_size( ch1) ;
-	size2 = ml_char_size( ch2) ;
-
-	if( size1 != size2)
+	if( ml_char_code( ch1) != ml_char_code( ch2))
 	{
 		return  0 ;
 	}
 
-	if( memcmp( ml_char_bytes( ch1) , ml_char_bytes( ch2) , size1) != 0)
-	{
-		return  0 ;
-	}
-	
 	comb1 = ml_get_combining_chars( ch1 , &comb1_size) ;
 	comb2 = ml_get_combining_chars( ch2 , &comb2_size) ;
 
@@ -973,7 +912,7 @@ ml_char_bytes_equal(
 
 	for( count = 0 ; count < comb1_size ; count ++)
 	{
-		if( ! ml_char_bytes_equal( &comb1[count] , &comb2[count]))
+		if( comb1[count].u.ch.code != comb2[count].u.ch.code)
 		{
 			return  0 ;
 		}
@@ -990,8 +929,8 @@ ml_sp_ch(void)
 	if( sp_ch.u.ch.attr == 0)
 	{
 		ml_char_init( &sp_ch) ;
-		ml_char_set( &sp_ch , (u_char *)" " , 1 , US_ASCII , 0 , 0 ,
-			ML_FG_COLOR , ML_BG_COLOR , 0 , 0) ;
+		ml_char_set( &sp_ch , ' ' , US_ASCII , 0 , 0 ,
+			ML_FG_COLOR , ML_BG_COLOR , 0 , 0 , 0) ;
 	}
 
 	return  &sp_ch ;
@@ -1005,8 +944,8 @@ ml_nl_ch(void)
 	if( nl_ch.u.ch.attr == 0)
 	{
 		ml_char_init( &nl_ch) ;
-		ml_char_set( &nl_ch , (u_char *)"\n" , 1 , US_ASCII , 0 , 0 ,
-			ML_FG_COLOR , ML_BG_COLOR , 0 , 0) ;
+		ml_char_set( &nl_ch , '\n' , US_ASCII , 0 , 0 ,
+			ML_FG_COLOR , ML_BG_COLOR , 0 , 0 , 0) ;
 	}
 
 	return  &nl_ch ;
@@ -1015,7 +954,7 @@ ml_nl_ch(void)
 
 #ifdef  DEBUG
 
-#if  1
+#if  0
 #define  DUMP_HEX
 #endif
 
@@ -1031,37 +970,26 @@ ml_char_dump(
 	ml_char_t *  comb_chars ;
 	
 #ifdef  DUMP_HEX
-	int  i ;
-	
-	kik_msg_printf( "[") ;
-	for( i = 0 ; i < ml_char_size(ch) ; i ++)
-	{
-		kik_msg_printf( "%.2x" , ml_char_bytes(ch)[i]) ;
-	}
-	kik_msg_printf( "]") ;
+	kik_msg_printf( "[%.4x]" , ml_char_code(ch)) ;
 #else
-	if( ml_char_size(ch) == 2)
+	if( ml_char_code(ch) >= 0x100)
 	{
 		if( ml_char_cs(ch) == JISX0208_1983)
 		{
 			/* only eucjp */
 
-			kik_msg_printf( "%c%c" , ml_char_bytes(ch)[0] | 0x80 ,
-				ml_char_bytes(ch)[1] | 0x80) ;
+			kik_msg_printf( "%c%c" ,
+				((ml_char_code(ch) >> 8) & 0xff) | 0x80 ,
+				(ml_char_code(ch) & 0xff) | 0x80) ;
 		}
 		else
 		{
 			kik_msg_printf( "**") ;
 		}
 	}
-	else if( ml_char_size(ch) == 1)
-	{
-		kik_msg_printf( "%c" , ml_char_bytes(ch)[0]) ;
-	}
 	else
 	{
-		kik_msg_printf( "!!! unsupported char[0x%.2x len %d] !!!" ,
-			ml_char_bytes(ch)[0] , SIZE(ch->u.ch.attr)) ;
+		kik_msg_printf( "%c" , ml_char_code(ch)) ;
 	}
 #endif
 
