@@ -1298,7 +1298,7 @@ ml_line_get_num_of_filled_chars_except_spaces(
 		{
 		#if  1
 			/* >= 3.0.6 */
-			if( ! ml_char_bytes_equal( line->chars + char_index , ml_sp_ch()))
+			if( ! ml_char_code_equal( line->chars + char_index , ml_sp_ch()))
 		#else
 			/* <= 3.0.5 */
 			if( ! ml_char_equal( line->chars + char_index , ml_sp_ch()))
