@@ -105,12 +105,16 @@ int  ml_screen_resize( ml_screen_t *  screen , u_int  cols , u_int  rows) ;
 #define  ml_screen_set_bce_bg_color( screen , bg_color) \
 		ml_edit_set_bce_bg_color( (screen)->edit , bg_color)
 
-#define  ml_screen_cursor_col( screen)  ml_cursor_col( (screen)->edit)
-
 #define  ml_screen_cursor_char_index( screen)  ml_cursor_char_index( (screen)->edit)
 
+#define  ml_screen_cursor_col( screen)  ml_cursor_col( (screen)->edit)
+
 #define  ml_screen_cursor_row( screen)  ml_cursor_row( (screen)->edit)
 
+#define  ml_screen_cursor_relative_col( screen)  ml_cursor_relative_col( (screen)->edit)
+
+#define  ml_screen_cursor_relative_row( screen)  ml_cursor_relative_row( (screen)->edit)
+
 int  ml_screen_cursor_row_in_screen( ml_screen_t *  screen) ;
 
 #define  ml_screen_get_cols( screen)  ml_edit_get_cols( (screen)->edit)
@@ -206,9 +210,10 @@ int  ml_screen_search_find( ml_screen_t *  screen , int *  beg_char_index , int
  
 ml_char_t *  ml_screen_get_n_prev_char( ml_screen_t *  screen , int  n) ;
 
-int  ml_screen_combine_with_prev_char( ml_screen_t *  screen , u_char *  bytes ,
-	size_t  ch_size , mkf_charset_t  cs , int  is_biwidth , int  is_comb ,
-	ml_color_t  fg_color , ml_color_t  bg_color , int  is_bold , int  is_underlined) ;
+int  ml_screen_combine_with_prev_char( ml_screen_t *  screen , u_int32_t  code ,
+	mkf_charset_t  cs , int  is_fullwidth , int  is_comb ,
+	ml_color_t  fg_color , ml_color_t  bg_color , int  is_bold ,
+	int  is_italic , int  is_underlined) ;
 
 int  ml_screen_insert_chars( ml_screen_t *  screen , ml_char_t *  chars , u_int  len) ;
 
@@ -250,6 +255,12 @@ int  ml_screen_delete_lines( ml_screen_t *  screen , u_int  size) ;
 #define  ml_screen_set_scroll_region( screen , beg , end) \
 		ml_edit_set_scroll_region( (screen)->edit , beg , end)
 
+#define  ml_screen_set_use_margin( screen , use) \
+		ml_edit_set_use_margin( (screen)->edit , use)
+
+#define  ml_screen_set_margin( screen , beg , end) \
+		ml_edit_set_margin( (screen)->edit , beg , end)
+
 #define  ml_screen_index( screen)  ml_edit_go_downward( (screen)->edit , SCROLL)
 
 #define  ml_screen_reverse_index( screen)  ml_edit_go_upward( (screen)->edit , SCROLL)
