@@ -78,6 +78,8 @@ ml_term_new(
 		goto  error ;
 	}
 
+	ml_vt100_parser_set_use_bidi( term->parser , use_bidi) ;
+
 	if( ! ml_config_menu_init( &term->config_menu))
 	{
 	#ifdef  DEBUG
@@ -137,6 +139,8 @@ ml_term_delete(
 		(*term->shape->delete)( term->shape) ;
 	}
 
+	free( term->uri) ;
+
 	free( term->icon_path) ;
 
 	ml_screen_delete( term->screen) ;
@@ -202,6 +206,11 @@ ml_term_open_pty(
 			return  0 ;
 		}
 
+		if( pass)
+		{
+			term->uri = strdup( host) ;
+		}
+
 		ml_term_plug_pty( term , pty) ;
 	}
 
@@ -311,6 +320,8 @@ ml_term_set_use_bidi(
 {
 	term->use_bidi = flag ;
 
+	ml_vt100_parser_set_use_bidi( term->parser , flag) ;
+
 	return  1 ;
 }
 
