@@ -129,6 +129,7 @@ typedef struct  ml_vt100_storable_states
 	int8_t  is_saved ;
 	
 	int8_t  is_bold ;
+	int8_t  is_italic ;
 	int8_t  is_underlined ;
 	int8_t  is_reversed ;
 	ml_color_t  fg_color ;
@@ -182,6 +183,7 @@ typedef struct  ml_vt100_parser
 	int8_t  is_so ;
 
 	int8_t  is_bold ;
+	int8_t  is_italic ;
 	int8_t  is_underlined ;
 	int8_t  is_reversed ;
 
@@ -206,10 +208,16 @@ typedef struct  ml_vt100_parser
 #endif
 	int8_t  modify_other_keys ;
 
+	int8_t  use_bidi ;
+
 #ifdef  USE_VT52
 	int8_t  is_vt52_mode ;
 #endif
 
+	int8_t  sixel_scrolling ;
+
+	int8_t  yield ;
+
 	/* for save/restore cursor */
 	ml_vt100_storable_states_t  saved_normal ;
 	ml_vt100_storable_states_t  saved_alternate ;
@@ -224,6 +232,10 @@ void  ml_set_use_alt_buffer( int  use) ;
 
 void  ml_set_use_ansi_colors( int  use) ;
 
+void  ml_set_use_scp( int  use) ;
+
+int  ml_set_unicode_noconv_areas( char *  areas) ;
+
 ml_vt100_parser_t *  ml_vt100_parser_new( ml_screen_t *  screen , ml_char_encoding_t  encoding ,
 	ml_unicode_policy_t  policy , u_int  col_size_a ,
 	int  use_char_combining , int  use_multi_col_char ,
@@ -307,6 +319,8 @@ int  ml_vt100_parser_set_unicode_policy( ml_vt100_parser_t *  vt100_parser ,
 
 #define  ml_vt100_parser_get_unicode_policy( vt100_parser)  ((vt100_parser)->unicode_policy)
 
+#define  ml_vt100_parser_set_use_bidi( vt100_parser , use)  ((vt100_parser)->use_bidi = (use))
+
 int  ml_convert_to_internal_ch( mkf_char_t *  ch , ml_unicode_policy_t  unicode_policy ,
 		mkf_charset_t  gl) ;
 
