@@ -19,7 +19,7 @@
 #include  <kiklib/kik_path.h>	/* kik_basename */
 #include  <kiklib/kik_conf_io.h>/* kik_get_user_rc_path */
 #include  <kiklib/kik_util.h>	/* TOINT32 */
-#include  <mkf/mkf_char.h>	/* mkf_bytes_to_int */
+#include  <mkf/mkf_char.h>
 
 
 #define  DIVIDE_ROUNDING(a,b)  ( ((int)((a)*10 + (b)*5)) / ((int)((b)*10)) )
@@ -291,7 +291,7 @@ load_encodings(
 static int
 get_metrics(
 	u_int8_t *  width ,
-	u_int8_t *  width_bi ,
+	u_int8_t *  width_full ,
 	u_int8_t *  height ,
 	u_int8_t *  ascent ,
 	u_char *  p ,
@@ -317,16 +317,16 @@ get_metrics(
 		{
 			/* U+3000: Unicode ideographic space (Full width) */
 			p += (5 * 0x3000) ;
-			*width_bi = p[2] - 0x80 ;
+			*width_full = p[2] - 0x80 ;
 		}
 		else
 		{
-			*width_bi = 0 ;
+			*width_full = 0 ;
 		}
 
 	#ifdef  __DEBUG
 		kik_debug_printf( KIK_DEBUG_TAG " COMPRESSED METRICS %d %d %d %d %d\n" ,
-			num_of_metrics , *width , *width_bi , *height , *ascent) ;
+			num_of_metrics , *width , *width_full , *height , *ascent) ;
 	#endif
 	}
 	else
@@ -356,16 +356,16 @@ get_metrics(
 			/* skip {left|right}_sided_bearing */
 			p += 4 ;
 
-			*width_bi = _TOINT16(p,is_be) ;
+			*width_full = _TOINT16(p,is_be) ;
 		}
 		else
 		{
-			*width_bi = 0 ;
+			*width_full = 0 ;
 		}
 
 	#ifdef  __DEBUG
 		kik_debug_printf( KIK_DEBUG_TAG " NOT COMPRESSED METRICS %d %d %d %d %d\n" ,
-			num_of_metrics , *width , *width_bi , *height , *ascent) ;
+			num_of_metrics , *width , *width_full , *height , *ascent) ;
 	#endif
 	}
 
@@ -439,18 +439,21 @@ gunzip(
 	 */
 	system( cmd) ;
 
+	/* st->st_size can be 0 if file_path points an illegally gzipped file. */
+	if( stat( new_file_path , st) == -1 || st->st_size <= 8)
+	{
+		unlink( new_file_path) ;
+
+		goto  error ;
+	}
+
 	/*
 	 * The atime and mtime of the uncompressed pcf font is the same
 	 * as those of the original gzipped font.
 	 */
 	ut.actime = st->st_atime ;
 	ut.modtime = st->st_mtime ;
-	if( utime( new_file_path , &ut) == -1)
-	{
-		goto  error ;
-	}
-
-	stat( new_file_path , st) ;
+	utime( new_file_path , &ut) ;
 
 #ifdef  __DEBUG
 	kik_debug_printf( KIK_DEBUG_TAG " USE NEWLY UNCOMPRESSED FONT\n") ;
@@ -509,6 +512,7 @@ load_pcf(
 
 	table_load_count = 0 ;
 
+	/* "st.st_size > 8" is ensured. (see gunzip()) */
 	if( ! ( p = pcf = mmap( NULL , st.st_size , PROT_READ , MAP_PRIVATE , fd , 0)) ||
 	    memcmp( p , "\1fcp" , 4) != 0)
 	{
@@ -520,6 +524,11 @@ load_pcf(
 	num_of_tables = _TOINT32(p,0) ;
 	p += 4 ;
 
+	if( st.st_size <= 8 + 16 * num_of_tables)
+	{
+		goto  end ;
+	}
+
 	for( count = 0 ; count < num_of_tables ; count++)
 	{
 		int32_t  type ;
@@ -568,7 +577,7 @@ load_pcf(
 		}
 		else if( type == PCF_METRICS)
 		{
-			if( ! get_metrics( &xfont->width , &xfont->width_bi , &xfont->height ,
+			if( ! get_metrics( &xfont->width , &xfont->width_full , &xfont->height ,
 					&xfont->ascent , pcf + offset + 4 , size ,
 					format & 4 , format & 0x100))
 			{
@@ -598,7 +607,7 @@ load_pcf(
 			{
 				u_char *  line ;
 
-				line = x_get_bitmap_line( xfont , bitmap , j) ;
+				x_get_bitmap_line( xfont , bitmap , j , line) ;
 
 				for( i = 0 ; i < xfont->width ; i++)
 				{
@@ -681,38 +690,99 @@ x_font_new(
 	)
 {
 	char *  font_file ;
-	char *  percent_str ;
 	u_int  percent ;
 	x_font_t *  font ;
 	void *  p ;
 	u_int  count ;
 
 	if( ! fontname)
-	{
-		kik_msg_printf( "Font file is not specified.\n") ;
-
-		return  NULL ;
-	}
-
-	if( type_engine != TYPE_XCORE || ! ( font = calloc( 1 , sizeof(x_font_t))))
-	{
-		return  NULL ;
-	}
-
-	if( ( percent_str = kik_str_alloca_dup( fontname)) == NULL)
 	{
 	#ifdef  DEBUG
-		kik_warn_printf( KIK_DEBUG_TAG " alloca() failed.\n") ;
+		kik_debug_printf( KIK_DEBUG_TAG " Font file is not specified.\n") ;
 	#endif
 
-		return  0 ;
+		if( FONT_CS(id) == ISO10646_UCS4_1 || FONT_CS(id) == ISO8859_1_R)
+		{
+			struct stat  st ;
+
+		#if  defined(__FreeBSD__)
+			if( stat( "/usr/local/lib/X11/fonts/local/unifont.pcf.gz" , &st) == 0)
+			{
+				font_file = "/usr/local/lib/X11/fonts/local/unifont.pcf.gz" ;
+				percent = 100 ;
+			}
+			else
+			{
+				font_file = "/usr/local/lib/X11/fonts/misc/10x20.pcf.gz" ;
+				percent = 0 ;
+			}
+		#elif  defined(__NetBSD__)
+			percent = 0 ;
+			if( stat( "/usr/pkg/lib/X11/fonts/efont/b16.pcf.gz" , &st) == 0)
+			{
+				font_file = "/usr/pkg/lib/X11/fonts/efont/b16.pcf.gz" ;
+			}
+			else
+			{
+				font_file = "/usr/X11R7/lib/X11/fonts/misc/10x20.pcf.gz" ;
+			}
+		#elif  defined(__OpenBSD__)
+			if( stat( "/usr/X11R6/lib/X11/fonts/misc/unifont.pcf.gz" , &st) == 0)
+			{
+				font_file = "/usr/X11R6/lib/X11/fonts/misc/unifont.pcf.gz" ;
+				percent = 100 ;
+			}
+			else
+			{
+				font_file = "/usr/X11R6/lib/X11/fonts/misc/10x20.pcf.gz" ;
+				percent = 0 ;
+			}
+		#elif  defined(__ANDROID__)
+			font_file = "/sdcard/.mlterm/unifont.pcf" ;
+			percent = 100 ;
+		#else /* __linux__ */
+			if( stat( "/usr/share/fonts/X11/misc/unifont.pcf.gz" , &st) == 0)
+			{
+				font_file = "/usr/share/fonts/X11/misc/unifont.pcf.gz" ;
+				percent = 100 ;
+			}
+			else
+			{
+				font_file = "/usr/share/fonts/X11/misc/10x20.pcf.gz" ;
+				percent = 0 ;
+			}
+		#endif
+
+			if( id & FONT_BOLD)
+			{
+				use_medium_for_bold = 1 ;
+			}
+		}
+		else
+		{
+			return  NULL ;
+		}
+	}
+	else
+	{
+		char *  percent_str ;
+
+		if( ! ( percent_str = kik_str_alloca_dup( fontname)))
+		{
+			return  NULL ;
+		}
+
+		font_file = kik_str_sep( &percent_str , ":") ;
+
+		if( ! percent_str || ! kik_str_to_uint( &percent , percent_str))
+		{
+			percent = 0 ;
+		}
 	}
 
-	font_file = kik_str_sep( &percent_str , ":") ;
-
-	if( ! percent_str || ! kik_str_to_uint( &percent , percent_str))
+ 	if( type_engine != TYPE_XCORE || ! ( font = calloc( 1 , sizeof(x_font_t))))
 	{
-		percent = 0 ;
+		return  NULL ;
 	}
 
 	for( count = 0 ; count < num_of_xfonts ; count++)
@@ -754,7 +824,7 @@ xfont_loaded:
 
 	font->id = id ;
 
-	if( font->id & FONT_BIWIDTH)
+	if( font->id & FONT_FULLWIDTH)
 	{
 		font->cols = 2 ;
 	}
@@ -793,10 +863,10 @@ xfont_loaded:
 		font->is_double_drawing = 0 ;
 	}
 
-	if( ( id & FONT_BIWIDTH) && FONT_CS(id) == ISO10646_UCS4_1 &&
-	    font->xfont->width_bi > 0)
+	if( ( id & FONT_FULLWIDTH) && FONT_CS(id) == ISO10646_UCS4_1 &&
+	    font->xfont->width_full > 0)
 	{
-		font->width = font->xfont->width_bi ;
+		font->width = font->xfont->width_full ;
 	}
 	else
 	{
@@ -1020,7 +1090,7 @@ x_change_font_cols(
 {
 	if( cols == 0)
 	{
-		if( font->id & FONT_BIWIDTH)
+		if( font->id & FONT_FULLWIDTH)
 		{
 			font->cols = 2 ;
 		}
@@ -1040,8 +1110,7 @@ x_change_font_cols(
 u_int
 x_calculate_char_width(
 	x_font_t *  font ,
-	const u_char *  ch ,
-	size_t  len ,
+	u_int32_t  ch ,
 	mkf_charset_t  cs ,
 	int *  draw_alone
 	)
@@ -1061,64 +1130,44 @@ x_calculate_char_width(
 	return  font->width ;
 }
 
-char **
-x_font_get_encoding_names(
-	mkf_charset_t  cs
-	)
-{
-	return  NULL ;
-}
-
 /* Return written size */
 size_t
 x_convert_ucs4_to_utf16(
 	u_char *  dst ,	/* 4 bytes. Big endian. */
-	u_char *  src	/* 4 bytes. Big endian. */
+	u_int32_t  src
 	)
 {
-#if  0
-	kik_debug_printf( KIK_DEBUG_TAG "%.8x => " , mkf_bytes_to_int( src , 4)) ;
-#endif
-
-	if( src[0] > 0x0 || src[1] > 0x10)
+	if( src < 0x10000)
 	{
-		return  0 ;
-	}
-	else if( src[1] == 0x0)
-	{
-		dst[0] = src[2] ;
-		dst[1] = src[3] ;
+		dst[0] = (src >> 8) & 0xff ;
+		dst[1] = src & 0xff ;
 
 		return  2 ;
 	}
-	else /* if( src[1] <= 0x10) */
+	else if( src < 0x110000)
 	{
 		/* surrogate pair */
 
-		u_int32_t  linear ;
 		u_char  c ;
 
-		linear = mkf_bytes_to_int( src , 4) - 0x10000 ;
-
-		c = (u_char)( linear / (0x100 * 0x400)) ;
-		linear -= (c * 0x100 * 0x400) ;
+		src -= 0x10000 ;
+		c = (u_char)( src / (0x100 * 0x400)) ;
+		src -= (c * 0x100 * 0x400) ;
 		dst[0] = c + 0xd8 ;
 
-		c = (u_char)( linear / 0x400) ;
-		linear -= (c * 0x400) ;
+		c = (u_char)( src / 0x400) ;
+		src -= (c * 0x400) ;
 		dst[1] = c ;
 
-		c = (u_char)( linear / 0x100) ;
-		linear -= (c * 0x100) ;
+		c = (u_char)( src / 0x100) ;
+		src -= (c * 0x100) ;
 		dst[2] = c + 0xdc ;
-		dst[3] = (u_char) linear ;
-
-	#if  0
-		kik_msg_printf( "%.2x%.2x%.2x%.2x\n" , dst[0] , dst[1] , dst[2] , dst[3]) ;
-	#endif
+		dst[3] = (u_char)src ;
 
 		return  4 ;
 	}
+
+	return  0 ;
 }
 
 
@@ -1171,16 +1220,22 @@ x_get_bitmap(
 	int16_t  glyph_idx ;
 	int32_t  glyph_offset ;
 
-	if( len == 2)
-	{
-		ch_idx = (ch[0] - xfont->min_byte1) *
-			 (xfont->max_char_or_byte2 - xfont->min_char_or_byte2 + 1) +
-			ch[1] - xfont->min_char_or_byte2 ;
-	}
-	else /* if( len == 1) */
+	if( len == 1)
 	{
 		ch_idx = ch[0] - xfont->min_char_or_byte2 ;
 	}
+	else if( len == 2)
+	{
+		ch_idx = (ch[0] - xfont->min_byte1) *
+		         (xfont->max_char_or_byte2 - xfont->min_char_or_byte2 + 1) +
+		         ch[1] - xfont->min_char_or_byte2 ;
+	}
+	else /* if( len == 4) */
+	{
+		ch_idx = (ch[1] * 0x100 + ch[2] - xfont->min_byte1) *
+		         (xfont->max_char_or_byte2 - xfont->min_char_or_byte2 + 1) +
+		         ch[3] - xfont->min_char_or_byte2 ;
+	}
 
 	if( ch_idx >= ( xfont->max_char_or_byte2 - xfont->min_char_or_byte2 + 1) *
 			( xfont->max_byte1 - xfont->min_byte1 + 1) ||
