@@ -20,6 +20,10 @@
 #define  BOTTOM_MARGIN(win) \
 	((win)->height_inc ? ((win)->height - (win)->min_height) % (win)->height_inc : 0)
 
+#ifdef  USE_GRF
+static x_color_t  black = { TP_COLOR , 0 , 0 , 0 , 0 } ;
+#endif
+
 
 /* --- static variables --- */
 
@@ -154,9 +158,45 @@ draw_string(
 
 	xfont = font->xfont ;
 
-	for( count = 0 ; count < len ; count++)
+	if( ch_len == 1)
 	{
-		bitmaps[count] = x_get_bitmap( xfont , str + count * ch_len , ch_len) ;
+		for( count = 0 ; count < len ; count++)
+		{
+			bitmaps[count] = x_get_bitmap( xfont , str + count , 1) ;
+		}
+	}
+	else /* if( ch_len == 2) */
+	{
+		for( count = 0 ; count < len ; count++)
+		{
+			if( 0xd8 <= str[0] && str[0] <= 0xdb)
+			{
+				if( count + 1 >= len)
+				{
+					break ;
+				}
+
+				if( 0xdc <= str[2] && str[2] <= 0xdf)
+				{
+					/* surrogate pair */
+
+					mkf_int_to_bytes( str , 4 ,
+						(str[0] - 0xd8) * 0x100 * 0x400 + str[1] * 0x400 +
+						(str[2] - 0xdc) * 0x100 + str[3] + 0x10000) ;
+					ch_len = 4 ;
+					len -- ;
+				}
+				else
+				{
+					str += 2 ;
+				}
+			}
+
+			bitmaps[count] = x_get_bitmap( xfont , str , ch_len) ;
+
+			str += ch_len ;
+			ch_len = 2 ;
+		}
 	}
 
 	/*
@@ -322,7 +362,7 @@ draw_string(
 					{
 						p += (font->x_off * 4) ;
 
-						for( x_off = 0 ; x_off < font->width ; x_off++)
+						for( x_off = 0 ; x_off < glyph_width ; x_off++)
 						{
 							if( x_get_bitmap_cell( bitmap_line ,
 									x_off))
@@ -456,7 +496,7 @@ copy_area(
 	u_int  height ,
 	int  dst_x ,	/* can be minus */
 	int  dst_y ,	/* can be minus */
-	int  accept_margin	/* x/y can be minus or not */
+	int  accept_margin	/* x/y can be minus and over width/height */
 	)
 {
 	int  margin ;
@@ -467,7 +507,7 @@ copy_area(
 	u_char *  picture ;
 	size_t  src_width_size ;
 
-	if( ! win->is_mapped || dst_x >= (int)win->width || dst_y >= (int)win->height)
+	if( ! win->is_mapped)
 	{
 		return  0 ;
 	}
@@ -484,6 +524,11 @@ copy_area(
 		bottom_margin = BOTTOM_MARGIN(win) ;
 	}
 
+	if( dst_x >= (int)win->width + margin || dst_y >= (int)win->height + margin)
+	{
+		return  0 ;
+	}
+
 	if( dst_x + width > win->width + margin - right_margin)
 	{
 		width = win->width + margin - right_margin - dst_x ;
@@ -614,7 +659,7 @@ window_move(
 	x_window_t *  win ,
 	int  x ,
 	int  y ,
-	int  expose	/* 1=full, -1=normal, 0=no */
+	int  expose	/* 1=normal, 0=no */
 	)
 {
 	int  prev_x ;
@@ -626,14 +671,14 @@ window_move(
 	win->x = x ;
 	win->y = y ;
 
-	if( expose == 1)
+	if( expose)
 	{
 		/*
-		 * x_expose_window() must be called after win->x and win->y is set
-		 * because window_exposed event which is called in x_expose_window()
-		 * can call x_window_move() recursively.
+		 * x_display_check_visibility_of_im_window() must be called after
+		 * win->x and win->y is set because window_exposed event which is
+		 * called in expose_window() can call x_window_move() recursively.
 		 */
-		x_display_expose( prev_x , prev_y , ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win)) ;
+		x_display_check_visibility_of_im_window() ;
 	}
 
 	clear_margin_area( win) ;
@@ -775,6 +820,30 @@ x_window_set_wall_picture(
 	Pixmap  pic
 	)
 {
+#ifdef  USE_GRF
+	int  ret ;
+
+	if( ( ret = x68k_tvram_set_wall_picture( pic->image , pic->width , pic->height)))
+	{
+		win->wall_picture = 0x1 ;	/* dummy */
+
+		/* Don't set is_scrollable = 0. */
+
+		/* If ret == 2, text vram was initialized just now. */
+		if( ret == 2)
+		{
+			clear_margin_area( win) ;
+
+			if( win->window_exposed)
+			{
+				(*win->window_exposed)( win , 0 , 0 , win->width , win->height) ;
+			}
+		}
+
+		return  0 ;	/* to free pic memory. */
+	}
+#endif
+
 	win->wall_picture = pic ;
 	win->is_scrollable = 0 ;
 
@@ -799,6 +868,10 @@ x_window_unset_wall_picture(
 	x_window_t *  win
 	)
 {
+#ifdef  USE_GRF
+	x68k_tvram_set_wall_picture( NULL , 0 , 0) ;
+#endif
+
 	win->wall_picture = None ;
 	win->is_scrollable = 1 ;
 
@@ -976,7 +1049,7 @@ x_window_show(
 		 * at the end of this function.
 		 */
 		is_mapped = win->is_mapped ;
-		win->is_mapped = 0 ;
+		win->is_mapped = 0 ;	/* XXX x_window_set_wall_picture() depends on this. */
 		(*win->window_realized)( win) ;
 		win->is_mapped = is_mapped ;
 	}
@@ -1082,8 +1155,9 @@ x_window_resize(
 		 * because wall_picture can be resized to fit to the new window
 		 * size in win->window_resized.
 		 *
-		 * Don't clear_margin_area() if flag == 0 because segfault happens
-		 * in selecting candidates on the input method window.
+		 * Don't clear_margin_area() if flag == 0 because x_window_resize()
+		 * is called before x_window_move() in x_im_*_screen.c and could
+		 * cause segfault.
 		 */
 		clear_margin_area( win) ;
 	}
@@ -1145,17 +1219,6 @@ x_window_move(
 	int  x ,
 	int  y
 	)
-{
-	return  window_move( win , x , y , -1) ;
-}
-
-/* XXX for x_im_status_screen */
-int
-x_window_move_full_expose(
-	x_window_t *  win ,
-	int  x ,
-	int  y
-	)
 {
 	return  window_move( win , x , y , 1) ;
 }
@@ -1180,6 +1243,13 @@ x_window_clear(
 	u_int  height
 	)
 {
+#ifdef  USE_GRF
+	if( x68k_tvram_is_enabled())
+	{
+		return  x_window_fill_with( win , &black , x , y , width , height) ;
+	}
+	else
+#endif
 	if( ! win->wall_picture)
 	{
 		return  x_window_fill_with( win , &win->bg_color , x , y , width , height) ;
@@ -1306,7 +1376,7 @@ x_window_update_all(
 
 	if( ! win->parent)
 	{
-		x_cmap_reset() ;
+		x_display_reset_cmap() ;
 	}
 
 	clear_margin_area( win) ;
@@ -1480,6 +1550,13 @@ x_window_get_str(
 
 	ch = event->ksym ;
 
+#ifdef  __ANDROID__
+	if( ch == 0)
+	{
+		return  x_display_get_str( seq , seq_len) ;
+	}
+#endif
+
 	if( ( *keysym = event->ksym) >= 0x100)
 	{
 		switch( *keysym)
@@ -1846,6 +1923,13 @@ x_window_draw_image_string(
 	u_int  len
 	)
 {
+#ifdef  USE_GRF
+	if( bg_color == NULL && x68k_tvram_is_enabled())
+	{
+		bg_color = &black ;
+	}
+#endif
+
 	return  draw_string( win , font , fg_color , bg_color , x , y , str , len , 1 ,
 			bg_color == NULL) ;
 }
@@ -1862,6 +1946,13 @@ x_window_draw_image_string16(
 	u_int  len
 	)
 {
+#ifdef  USE_GRF
+	if( bg_color == NULL && x68k_tvram_is_enabled())
+	{
+		bg_color = &black ;
+	}
+#endif
+
 	return  draw_string( win , font , fg_color , bg_color , x , y , str , len , 2 ,
 			bg_color == NULL) ;
 }
