@@ -16,7 +16,6 @@
 #include  <kiklib/kik_debug.h>
 #include  <kiklib/kik_mem.h>	/* alloca */
 #include  <kiklib/kik_str.h>	/* kik_str_sep/kik_str_to_int/memset/strncasecmp */
-#include  <mkf/mkf_char.h>	/* mkf_bytes_to_int */
 #include  <ml_char.h>		/* UTF_MAX_SIZE */
 
 
@@ -565,8 +564,8 @@ ft_font_open(
 	}
 }
 
-u_int  xft_calculate_char_width( x_font_t *  font , const u_char *  ch , size_t  len) ;
-u_int  cairo_calculate_char_width( x_font_t *  font , const u_char *  ch , size_t  len) ;
+u_int  xft_calculate_char_width( x_font_t *  font , u_int32_t  ch) ;
+u_int  cairo_calculate_char_width( x_font_t *  font , u_int32_t  ch) ;
 
 static int
 fc_set_font(
@@ -574,7 +573,6 @@ fc_set_font(
 	const char *  fontname ,
 	u_int  fontsize ,
 	u_int  col_width ,	/* if usascii font wants to be set , 0 will be set. */
-	int  use_medium_for_bold ,	/* Not used for now. */
 	u_int  letter_space ,
 	int  aa_opt ,		/* 0 = default , 1 = enable , -1 = disable */
 	int  use_xft
@@ -605,7 +603,25 @@ fc_set_font(
 		weight = -1 ;	/* use default value */
 	}
 
-	slant = -1 ;	/* use default value */
+	if( font->id & FONT_ITALIC)
+	{
+	#ifdef  USE_TYPE_XFT
+		/*
+		 * XXX
+		 * FC_CHAR_WIDTH=ch_width and FC_SPACING=FC_MONO make
+		 * the width of italic kochi font double of ch_width
+		 * on xft if slant >= 0. (They works fine for Dejavu
+		 * Sans Mono Italic, though.)
+		 */
+		font->is_var_col_width = 1 ;
+	#endif
+
+		slant = FC_SLANT_ITALIC ;
+	}
+	else
+	{
+		slant = -1 ;	/* use default value */
+	}
 
 	/*
 	 * x_off related to percent is set before ft_font_open while
@@ -634,6 +650,20 @@ fc_set_font(
 		if( parse_fc_font_name( &font_family , &weight , &slant , &fontsize_d ,
 						&font_encoding , &percent , p))
 		{
+		#ifdef  USE_TYPE_XFT
+			/*
+			 * XXX
+			 * FC_CHAR_WIDTH=ch_width and FC_SPACING=FC_MONO make
+			 * the width of italic kochi font double of ch_width
+			 * on xft if slant >= 0. (They works fine for Dejavu
+			 * Sans Mono Italic, though.)
+			 */
+			if( slant >= 0)
+			{
+				font->is_var_col_width = 1 ;
+			}
+		#endif
+
 			if( col_width == 0)
 			{
 				/* basic font (e.g. usascii) width */
@@ -686,11 +716,12 @@ fc_set_font(
 			}
 
 		#ifdef  DEBUG
-			kik_debug_printf( "Loading font %s%s%s %f %d\n" , font_family ,
+			kik_debug_printf( "Loading font %s%s%s %f %d%s\n" , font_family ,
 				weight == FC_WEIGHT_BOLD ? ":Bold" :
 					weight == FC_WEIGHT_LIGHT ? " Light" : "" ,
 				slant == FC_SLANT_ITALIC ? ":Italic" : "" ,
-				fontsize_d , font->is_var_col_width) ;
+				fontsize_d , ch_width ,
+				font->is_var_col_width ? "(varcolwidth)" : "") ;
 		#endif
 
 			if( ( xfont = ft_font_open( font , font_family , fontsize_d ,
@@ -788,7 +819,7 @@ font_found:
 			 * Proportional (font->is_var_col_width is true)
 			 * letter_space is ignored in variable column width mode.
 			 */
-			font->width = xft_calculate_char_width( font , "W" , 1) ;
+			font->width = xft_calculate_char_width( font , 'W') ;
 		}
 		else
 		{
@@ -828,19 +859,13 @@ font_found:
 		font->height = DOUBLE_ROUNDUP_TO_INT(extents.height) ;
 		font->ascent = DOUBLE_ROUNDUP_TO_INT(extents.ascent) ;
 
-		/*
-		 * Set letter_space here because cairo_font_open() ignores it.
-		 * (FC_CHAR_WIDTH doesn't make effect in cairo.)
-		 * Then, column width for vertical mode should be set again here.
-		 */
-
 		if( font->cols == 2)
 		{
 			font->width = DOUBLE_ROUNDUP_TO_INT(extents.max_x_advance) ;
 		}
 		else
 		{
-			font->width = cairo_calculate_char_width( font , "W" , 1) ;
+			font->width = cairo_calculate_char_width( font , 'W') ;
 
 			if( font->is_vertical)
 			{
@@ -850,6 +875,12 @@ font_found:
 			}
 		}
 
+		/*
+		 * Set letter_space here because cairo_font_open() ignores it.
+		 * (FC_CHAR_WIDTH doesn't make effect in cairo.)
+		 * Then, column width for vertical mode should be set again here.
+		 */
+
 		/* letter_space is ignored in variable column width mode. */
 		if( ! font->is_var_col_width && letter_space > 0)
 		{
@@ -916,7 +947,6 @@ xft_set_font(
 	const char *  fontname ,
 	u_int  fontsize ,
 	u_int  col_width ,		/* if usascii font wants to be set , 0 will be set. */
-	int  use_medium_for_bold ,	/* Not used for now. */
 	u_int  letter_space ,
 	int  aa_opt ,			/* 0 = default , 1 = enable , -1 = disable */
 	int  use_point_size ,
@@ -934,8 +964,7 @@ xft_set_font(
 	
 	dpi_for_fc = dpi ;
 
-	return  fc_set_font( font , fontname , fontsize , col_width , use_medium_for_bold ,
-			letter_space , aa_opt , 1) ;
+	return  fc_set_font( font , fontname , fontsize , col_width , letter_space , aa_opt , 1) ;
 }
 
 int
@@ -952,39 +981,21 @@ xft_unset_font(
 u_int
 xft_calculate_char_width(
 	x_font_t *  font ,
-	const u_char *  ch ,		/* US-ASCII or Unicode */
-	size_t  len
+	u_int32_t  ch		/* US-ASCII or Unicode */
 	)
 {
 	XGlyphInfo  extents ;
 
-	if( len == 1)
+	if( ch < 0x100)
 	{
-		XftTextExtents8( font->display , font->xft_font , ch , 1 , &extents) ;
-	}
-#if  0
-	else if( len == 2)
-	{
-		FcChar16  c ;
+		u_char  c ;
 
-		c = mkf_bytes_to_int( ch , len) ;
-		XftTextExtents16( font->display , font->xft_font , &c , 1 , &extents) ;
-	}
-#endif
-	else if( len == 4)
-	{
-		FcChar32  c ;
-
-		c = mkf_bytes_to_int( ch , 4) ;
-		XftTextExtents32( font->display , font->xft_font , &c , 1 , &extents) ;
+		c = ch ;
+		XftTextExtents8( font->display , font->xft_font , &c , 1 , &extents) ;
 	}
 	else
 	{
-	#ifdef  DEBUG
-		kik_warn_printf( KIK_DEBUG_TAG " char size %d is too large.\n" , len) ;
-	#endif
-
-		return  0 ;
+		XftTextExtents32( font->display , font->xft_font , &ch , 1 , &extents) ;
 	}
 
 	if( extents.xOff < 0)
@@ -1008,7 +1019,6 @@ cairo_set_font(
 	const char *  fontname ,
 	u_int  fontsize ,
 	u_int  col_width ,		/* if usascii font wants to be set , 0 will be set. */
-	int  use_medium_for_bold ,	/* Not used for now. */
 	u_int  letter_space ,
 	int  aa_opt ,			/* 0 = default , 1 = enable , -1 = disable */
 	int  use_point_size ,
@@ -1025,8 +1035,7 @@ cairo_set_font(
 	}
 	dpi_for_fc = dpi ;
 
-	return  fc_set_font( font , fontname , fontsize , col_width , use_medium_for_bold ,
-			letter_space , aa_opt , 0) ;
+	return  fc_set_font( font , fontname , fontsize , col_width , letter_space , aa_opt , 0) ;
 }
 
 int
@@ -1107,15 +1116,14 @@ x_convert_ucs4_to_utf8(
 u_int
 cairo_calculate_char_width(
 	x_font_t *  font ,
-	const u_char *  ch ,
-	size_t  len
+	u_int32_t  ch
 	)
 {
 	u_char  utf8[UTF_MAX_SIZE + 1] ;
 	cairo_text_extents_t  extents ;
 	int  width ;
 
-	utf8[ x_convert_ucs4_to_utf8( utf8 , mkf_bytes_to_int( ch , len))] = '\0' ;
+	utf8[ x_convert_ucs4_to_utf8( utf8 , ch)] = '\0' ;
 	cairo_scaled_font_text_extents( font->cairo_font , utf8 , &extents) ;
 
 #if  0
