@@ -610,7 +610,7 @@ text_out(
 	int  y ,
 	u_char *  str ,
 	u_int  len ,
-	mkf_charset_t  cs
+	mkf_charset_t  cs	/* FONT_CS(font->id) */
 	)
 {
 	if( cs == ISO10646_UCS4_1)
@@ -1667,15 +1667,16 @@ x_window_update(
 {
 	if( win->update_window_flag)
 	{
+		/* WM_APP_PAINT has been already posted. */
 		win->update_window_flag |= flag ;
-		
+
 		return  1 ;
 	}
 	else
 	{
 		win->update_window_flag = flag ;
 	}
-	
+
 	/*
 	 * WM_APP_PAINT message is posted only when update_window_flag is 0.
 	 */
@@ -1764,12 +1765,6 @@ x_window_receive_event(
 		return  0 ;
 	}
 
-	/*
-	 * Dispatch event to all window in case key was pressed in scrollbar window but
-	 * should be shown in text area.
-	 */
-	x_xic_filter_event( x_get_root_window( win), event) ;
-
 #ifndef  DISABLE_XDND
 	if( x_dnd_filter_event( event, win))
 	{
@@ -1897,8 +1892,14 @@ x_window_receive_event(
 			return  1 ;
 		}
 
-	case WM_SYSKEYDOWN:
 	case WM_KEYDOWN:
+		/*
+		 * Dispatch event to all windows in case key was pressed in the
+		 * scrollbar window but should be shown in the text area.
+		 */
+		x_xic_filter_event( x_get_root_window( win), event) ;
+
+	case WM_SYSKEYDOWN:
 		if( win->key_pressed)
 		{
 			XKeyEvent  kev ;
@@ -1912,18 +1913,28 @@ x_window_receive_event(
 			{
 				kev.ch = event->wparam ;
 
-				if( ( kev.state & ModMask) && event->msg == WM_SYSKEYDOWN)
+				if( kev.state & ModMask)
 				{
-					/* Alt+key (which doesn't cause WM_*_CHAR message) */
-
-					if( 'A' <= kev.ch && kev.ch <= 'Z')
+					if( event->msg == WM_SYSKEYDOWN)
 					{
 						/*
-						 * Upper case => Lower case.
-						 * event->wparam is always upper case
-						 * if alt key is pressed together.
+						 * Alt+key (which doesn't cause WM_*_CHAR message)
 						 */
-						kev.ch = event->wparam + 0x20 ;
+
+						if( 'A' <= kev.ch && kev.ch <= 'Z')
+						{
+							/*
+							 * Upper case => Lower case.
+							 * event->wparam is always upper case
+							 * if alt key is pressed together.
+							 */
+							kev.ch += 0x20 ;
+						}
+					}
+					else
+					{
+						/* wait for WM_*_CHAR message. */
+						break ;
 					}
 				}
 				else if( ( kev.state & ControlMask) &&
@@ -1965,8 +1976,10 @@ x_window_receive_event(
 				/*
 				 * - See x_xic_get_str() in win32/x_xic.c.
 				 * - Following keys don't cause WM_*_CHAR message.
+				 * - Ctrl+Alt+key(AltGr+key) can cause WM_*_CHAR message later.
 				 */
-				if( ( kev.state & ControlMask) &&
+				if( ! ( kev.state & ModMask) &&
+				    ( kev.state & ControlMask) &&
 				    ( event->wparam == VK_OEM_2 ||	/* Ctl+/ */
 				      event->wparam == VK_OEM_3 ||	/* Ctl+@ */
 				      event->wparam == VK_OEM_7 ||	/* Ctl+^ */
@@ -1980,13 +1993,15 @@ x_window_receive_event(
 				}
 			}
 
+			if( event->msg == WM_SYSKEYDOWN)
+			{
+				event->msg = WM_KEYDOWN ;
+				x_xic_filter_event( x_get_root_window( win), event) ;
+			}
+
 			(*win->key_pressed)( win , &kev) ;
 
-			/* Don't process VK_F10 as a menu key by DefWindowProc(). */
-			if( event->wparam == VK_F10)
-			{
-				return  1 ;
-			}
+			return  1 ;
 		}
 
 		/* Continue default processing. */
@@ -2000,6 +2015,12 @@ x_window_receive_event(
 
 			kev.state = get_key_state() ;
 
+			if( ( kev.state & (ControlMask|ModMask)) && event->wparam > 0x20)
+			{
+				/* Ctrl+Alt+key => AltGr+key */
+				kev.state &= ~(ControlMask|ModMask) ;
+			}
+
 		#if  0
 			/*
 			 * XXX
@@ -2255,7 +2276,13 @@ x_window_receive_event(
 
 			if( win->button_is_pressing)
 			{
-				if( win->button_motion)
+				/*
+				 * prev_button_press_event.{x|y} can be the same
+				 * as mev.{x|y} when window is pressed and focused.
+				 */
+				if( win->button_motion &&
+				    ( win->prev_button_press_event.x != mev.x ||
+				      win->prev_button_press_event.y != mev.y))
 				{
 					(*win->button_motion)( win , &mev) ;
 				}
@@ -2321,11 +2348,18 @@ x_window_receive_event(
 		return  1 ;
 
 	case  WM_DESTROYCLIPBOARD:
-		/*
-		 * Call win->selection_cleared and win->is_sel_owner is set 0
-		 * in x_display_clear_selection.
-		 */
-		x_display_clear_selection( win->disp , win) ;
+		if( win->is_sel_owner == 1)
+		{
+			/*
+			 * Call win->selection_cleared and win->is_sel_owner is set 0
+			 * in x_display_clear_selection.
+			 */
+			x_display_clear_selection( win->disp , win) ;
+		}
+		else if( win->is_sel_owner > 1)
+		{
+			win->is_sel_owner -- ;
+		}
 
 		return  1 ;
 
@@ -2355,18 +2389,22 @@ x_window_receive_event(
 			u_int  height ;
 			u_int  min_width ;
 			u_int  min_height ;
+			u_int  old_width ;
+			u_int  old_height ;
 
 			width = LOWORD(event->lparam) ;
 			height = HIWORD(event->lparam) ;
 
-		#if  0
-			kik_debug_printf( "WM_SIZE resized from %d %d to %d %d\n" ,
-				ACTUAL_WIDTH(win), ACTUAL_HEIGHT(win), width, height) ;
-		#endif
-		
 			min_width = total_min_width(win) ;
 			min_height = total_min_height(win) ;
+			old_width = ACTUAL_WIDTH(win) ;
+			old_height = ACTUAL_HEIGHT(win) ;
 
+		#if  0
+			kik_debug_printf( "WM_SIZE resized from %d %d to %d %d\n" ,
+				old_width, old_height, width, height) ;
+		#endif
+		
 			if( width < min_width || height < min_height)
 			{
 				x_window_resize( win,
@@ -2374,32 +2412,32 @@ x_window_receive_event(
 					K_MAX(min_height,height) - win->margin * 2,
 					NOTIFY_TO_MYSELF) ;
 			}
-			else if( width != ACTUAL_WIDTH(win) || height != ACTUAL_HEIGHT(win))
+			else if( width != old_width || height != old_height)
 			{
 				u_int  width_surplus ;
 				u_int  height_surplus ;
 
-				if( width > ACTUAL_WIDTH(win))
+				if( width >= old_width)
 				{
-					width_surplus = (width - ACTUAL_WIDTH(win)) %
+					width_surplus = (width - old_width) %
 							total_width_inc(win) ;
 				}
 				else
 				{
 					width_surplus = total_width_inc(win) -
-							( (ACTUAL_WIDTH(win) - width) %
+							( (old_width - width) %
 							  total_width_inc(win) ) ;
 				}
 
-				if( height > ACTUAL_HEIGHT(win))
+				if( height >= old_height)
 				{
-					height_surplus = (height - ACTUAL_HEIGHT(win)) %
+					height_surplus = (height - old_height) %
 							total_height_inc(win) ;
 				}
 				else
 				{
 					height_surplus = total_height_inc(win) -
-							( (ACTUAL_HEIGHT(win) - height) %
+							( (old_height - height) %
 							  total_height_inc(win) ) ;
 				}
 				
@@ -2650,7 +2688,7 @@ x_window_copy_area(
 	{
 		MaskBlt( win->gc->gc , win->margin + dst_x , win->margin + dst_y ,
 			width , height , src , src_x , src_y , mask , src_x , src_y ,
-			MAKEROP4(SRCCOPY,PATCOPY)) ;
+			MAKEROP4(SRCCOPY,0x00aa0029)) ;
 	}
 	else
 	{
@@ -2904,30 +2942,31 @@ x_window_set_selection_owner(
 	Time  time
 	)
 {
-	/*
-	 * Whether win->is_sel_owner is 1 or 0, set clipboard NULL
-	 * whenever x_window_set_selection_owner is called.
-	 */
-	 
-	if( OpenClipboard( win->my_window) == FALSE)
-	{
-		win->is_sel_owner = 0 ;
-		
-		return  0 ;
-	}
-	
 #if  0
 	kik_debug_printf( KIK_DEBUG_TAG " x_window_set_selection_owner.\n") ;
 #endif
 
-	EmptyClipboard() ;
+	if( OpenClipboard( win->my_window) == FALSE ||
+	    ( ! win->is_sel_owner &&
+	      ! x_display_own_selection( win->disp , win)))
+	{
+		return  0 ;
+	}
+
+	/*
+	 * If win->is_sel_owner is already 1, win->is_sel_owner++ prevents
+	 * WM_DESTROYCLIPBOARD by EmtpyClipboard() from calling
+	 * x_display_clear_selection().
+	 */
+	win->is_sel_owner ++ ;
+
+	EmptyClipboard() ;	/* Own clipboard. Create WM_DESTROYCLIPBOARD message */
 	SetClipboardData( CF_TEXT, NULL) ;
 	SetClipboardData( CF_UNICODETEXT, NULL) ;
+
 	CloseClipboard() ;
 
-	win->is_sel_owner = 1 ;
-	
-	return  x_display_own_selection( win->disp , win) ;
+	return  1 ;
 }
 
 int
@@ -2941,21 +2980,23 @@ x_window_xct_selection_request(
 	u_char *  g_data ;
 	size_t  len ;
 	
-	if( IsClipboardFormatAvailable( CF_TEXT) == FALSE)
+	if( IsClipboardFormatAvailable( CF_TEXT) == FALSE ||
+	    OpenClipboard( win->my_window) == FALSE ||
+	    ( hmem = GetClipboardData( CF_TEXT)) == NULL)
 	{
 		return  0 ;
 	}
 
-	OpenClipboard( win->my_window) ;
-	hmem = GetClipboardData( CF_TEXT) ;
-
-	g_data = GlobalLock( hmem) ;
-	len = strlen( g_data) ;
-	if( ( l_data = alloca( len + 1)) == NULL)
+	if( ( g_data = GlobalLock( hmem)) == NULL ||
+	    ( l_data = alloca( (len = strlen( g_data)) + 1)) == NULL)
 	{
-		GlobalUnlock( hmem) ;
+		if( g_data)
+		{
+			GlobalUnlock( hmem) ;
+		}
+
 		CloseClipboard() ;
-		
+
 		return  0 ;
 	}
 
@@ -2984,19 +3025,21 @@ x_window_utf_selection_request(
 	u_char *  g_data ;
 	size_t  len ;
 	
-	if( IsClipboardFormatAvailable( CF_UNICODETEXT) == FALSE)
+	if( IsClipboardFormatAvailable( CF_UNICODETEXT) == FALSE ||
+	    OpenClipboard( win->my_window) == FALSE ||
+	    ( hmem = GetClipboardData( CF_UNICODETEXT)) == NULL)
 	{
 		return  0 ;
 	}
 
-	OpenClipboard( win->my_window) ;
-	hmem = GetClipboardData( CF_UNICODETEXT) ;
-
-	g_data = GlobalLock( hmem) ;
-	len = lstrlenW( g_data) ;
-	if( ( l_data = alloca( (len + 1) * 2)) == NULL)
+	if( ( g_data = GlobalLock( hmem)) == NULL ||
+	    ( l_data = alloca( ( (len = lstrlenW( g_data)) + 1) * 2)) == NULL)
 	{
-		GlobalUnlock( hmem) ;
+		if( g_data)
+		{
+			GlobalUnlock( hmem) ;
+		}
+
 		CloseClipboard() ;
 		
 		return  0 ;
@@ -3037,7 +3080,12 @@ x_window_send_selection(
 		return  0 ;
 	}
 
-	g_data = GlobalLock( hmem) ;
+	if( ( g_data = GlobalLock( hmem)) == NULL)
+	{
+		GlobalFree( hmem) ;
+
+		return  0 ;
+	}
 
 	for( count = 0 ; count < sel_len ; count ++)
 	{
