@@ -40,7 +40,7 @@ get_spot(
 	}
 
 	spot->x = x + win->margin ;
-	spot->y = y /* + win->margin */ ;
+	spot->y = y + win->margin ;
 
 	return  1 ;
 }
@@ -81,6 +81,8 @@ x_xic_activate(
 	win->xic->ic = ImmGetContext( win->my_window) ;
 	win->xic->prev_keydown_wparam = 0 ;
 
+	x_xic_font_set_changed( win) ;
+
 	return  1 ;
 }
 
@@ -139,6 +141,15 @@ x_xic_font_set_changed(
 	x_window_t *  win
 	)
 {
+	if( HAS_XIM_LISTENER(win,get_fontset))
+	{
+		if( ImmSetCompositionFont( win->xic->ic ,
+			(*win->xim_listener->get_fontset)( win->xim_listener->self)))
+		{
+			return  1 ;
+		}
+	}
+
 	return  0 ;
 }
 
@@ -171,7 +182,7 @@ x_xic_set_spot(
 	MapWindowPoints( win->my_window, x_get_root_window( win)->my_window, &spot, 1) ;
 
 	cf.ptCurrentPos = spot ;
-	cf.dwStyle = CFS_POINT ;
+	cf.dwStyle = CFS_FORCE_POSITION ;
 
 	return  ImmSetCompositionWindow( win->xic->ic, &cf) ;
 }
@@ -299,8 +310,7 @@ x_xic_filter_event(
 {
 	u_int  count ;
 
-	if( event->msg != WM_KEYDOWN &&
-	    ( event->msg != WM_SYSKEYDOWN || event->wparam != VK_F10) )
+	if( event->msg != WM_KEYDOWN)
 	{
 		return  0 ;
 	}
@@ -325,6 +335,9 @@ x_xic_set_focus(
 	x_window_t *  win
 	)
 {
+	/* The composition font can be changed by the connection dialog box. */
+	x_xic_font_set_changed( win) ;
+
 	return  1 ;
 }
 
