@@ -102,11 +102,30 @@ receive_next_event(void)
 	#endif
 		tval.tv_sec = 0 ;
 
+		displays = x_get_opened_displays( &num_of_displays) ;
+
+	#ifdef  USE_LIBSSH2
+		if( ( ret = ml_pty_ssh_poll( &read_fds)) > 0)
+		{
+			break ;
+		}
+	#endif
+
 		maxfd = 0 ;
 		FD_ZERO( &read_fds) ;
 
-		displays = x_get_opened_displays( &num_of_displays) ;
-		
+	#ifdef  USE_LIBSSH2
+		for( count = 0 ; count < num_of_xssh_fds ; count ++)
+		{
+			FD_SET( xssh_fds[count] , &read_fds) ;
+
+			if( xssh_fds[count] > maxfd)
+			{
+				maxfd = xssh_fds[count] ;
+			}
+		}
+	#endif
+
 		for( count = 0 ; count < num_of_displays ; count ++)
 		{
 			x_display_receive_next_event( displays[count]) ;
@@ -131,17 +150,6 @@ receive_next_event(void)
 				maxfd = ptyfd ;
 			}
 		}
-		
-	#ifdef  USE_LIBSSH2
-		for( count = 0 ; count < num_of_xssh_fds ; count++)
-		{
-			FD_SET( xssh_fds[count] , &read_fds) ;
-			if( xssh_fds[count] > maxfd)
-			{
-				maxfd = xssh_fds[count] ;
-			}
-		}
-	#endif
 
 		for( count = 0 ; count < num_of_additional_fds ; count++)
 		{
@@ -200,9 +208,6 @@ receive_next_event(void)
 	 * Processing order should be as follows.
 	 *
 	 * X Window -> PTY -> additional_fds
-	 *
-	 * (ml_pty_ssh_send_recv_x11() should be called after ml_term_parse_vt100_sequence()
-	 * which can process x11 forwarding channel packets.)
 	 */
 
 	for( count = 0 ; count < num_of_displays ; count ++)
@@ -213,6 +218,18 @@ receive_next_event(void)
 		}
 	}
 
+#ifdef  USE_LIBSSH2
+	/*
+	 * ml_pty_ssh_send_recv_x11() should be called before
+	 * ml_term_parse_vt100_sequence() where xssh_fds can be deleted.
+	 */
+	for( count = num_of_xssh_fds ; count > 0 ; count--)
+	{
+		ml_pty_ssh_send_recv_x11( count - 1 ,
+			FD_ISSET( xssh_fds[count - 1] , &read_fds)) ;
+	}
+#endif
+
 	for( count = 0 ; count < num_of_terms ; count ++)
 	{
 		if( FD_ISSET( ml_term_get_master_fd( terms[count]) , &read_fds))
@@ -220,16 +237,6 @@ receive_next_event(void)
 			ml_term_parse_vt100_sequence( terms[count]) ;
 		}
 	}
-	
-#ifdef  USE_LIBSSH2
-	for( count = 0 ; count < num_of_xssh_fds ; count++)
-	{
-		if( FD_ISSET( xssh_fds[count] , &read_fds))
-		{
-			ml_pty_ssh_send_recv_x11( count) ;
-		}
-	}
-#endif
 
 	for( count = 0 ; count < num_of_additional_fds ; count++)
 	{
@@ -243,8 +250,6 @@ receive_next_event(void)
 			}
 		}
 	}
-
-	return ;
 }
 
 #endif
@@ -266,10 +271,6 @@ x_event_source_init(void)
 int
 x_event_source_final(void)
 {
-#if  ! defined(USE_WIN32GUI) && ! defined(DEBUG)
-	exit(0) ;
-#endif
-
 #ifndef  USE_WIN32API
 	free( additional_fds) ;
 #endif
@@ -285,6 +286,7 @@ x_event_source_process(void)
 	x_display_t **  displays ;
 	ml_term_t **  terms ;
 	u_int  num_of_terms ;
+	int *  xssh_fds ;
 	u_int  count ;
 #endif
 
@@ -306,6 +308,13 @@ x_event_source_process(void)
 	 * If pty is closed after ml_close_dead_terms() ...
 	 */
 
+#ifdef  USE_LIBSSH2
+	for( count = ml_pty_ssh_get_x11_fds( &xssh_fds) ; count > 0 ; count--)
+	{
+		ml_pty_ssh_send_recv_x11( count - 1 , 1) ;
+	}
+#endif
+
 	num_of_terms = ml_get_all_terms( &terms) ;
 
 	for( count = 0 ; count < num_of_terms ; count++)
