@@ -103,8 +103,7 @@ candidate_width(
 
 		width += x_calculate_char_width(
 					xfont ,
-					ml_char_bytes( &candidate->chars[i]) ,
-					ml_char_size( &candidate->chars[i]) ,
+					ml_char_code( &candidate->chars[i]) ,
 					ml_char_cs( &candidate->chars[i]) , NULL) ;
 	}
 
@@ -201,35 +200,40 @@ resize(
 	int  x ;
 	int  y ;
 
-	x_window_resize( &cand_screen->window , width , height , 0) ;
-
-	x = cand_screen->x ;
-	y = cand_screen->y ;
-	adjust_window_position_by_size( cand_screen , &x , &y) ;
-
-	if( x != cand_screen->window.x || y != cand_screen->window.y)
+	if( x_window_resize( &cand_screen->window , width , height , 0))
 	{
-		/*
-		 * In frmebuffer,
-		 * x_window_move() -> x_im_candidate_screen::window_exposed() ->
-		 * x_im_candidate_screen::draw_screen() -> here again
-		 */
+		x = cand_screen->x ;
+		y = cand_screen->y ;
+		adjust_window_position_by_size( cand_screen , &x , &y) ;
+
+		if( x != cand_screen->window.x || y != cand_screen->window.y)
+		{
+		#ifdef  USE_FRAMEBUFFER
+			/*
+			 * In frmebuffer,
+			 * x_im_candidate_screen::draw_screen() ->
+			 * x_im_candidate_screen::resize() ->
+			 * x_window_move() ->
+			 * x_im_candidate_screen::window_exposed() ->
+			 * x_im_candidate_screen::draw_screen()
+			 */
+			x_window_move_no_expose( &cand_screen->window , x , y) ;
+		#else
+			x_window_move( &cand_screen->window , x , y) ;
+		#endif
+		}
 	#ifdef  USE_FRAMEBUFFER
-		x_window_move_no_expose( &cand_screen->window , x , y) ;
-	#else
-		x_window_move( &cand_screen->window , x , y) ;
+		else
+		{
+			/* resized but position is not changed. */
+			x_window_clear_margin_area( &cand_screen->window) ;	/* XXX */
+		}
+
+		x_window_draw_rect_frame( &cand_screen->window , -MARGIN , -MARGIN ,
+					  cand_screen->window.width + MARGIN - 1 ,
+					  cand_screen->window.height + MARGIN - 1) ;
 	#endif
 	}
-#ifdef  USE_FRAMEBUFFER
-	else
-	{
-		x_window_clear_margin_area( &cand_screen->window) ;	/* XXX */
-	}
-
-	x_window_draw_rect_frame( &cand_screen->window , -MARGIN , -MARGIN ,
-				  cand_screen->window.width + MARGIN - 1 ,
-				  cand_screen->window.height + MARGIN - 1) ;
-#endif
 }
 
 #define  MAX_NUM_OF_DIGITS 4 /* max is 9999. enough? */
@@ -357,10 +361,9 @@ draw_screen_vertical(
 
 			ml_char_init( &ch) ;
 
-			ml_char_set( &ch , &digit[j] , 1 ,
-				     US_ASCII , 0 , 0 ,
+			ml_char_set( &ch , digit[j] , US_ASCII , 0 , 0 ,
 				     ML_FG_COLOR , ML_BG_COLOR ,
-				     0 , 0) ;
+				     0 , 0 , 0) ;
 
 			x_draw_str( &cand_screen->window ,
 				    cand_screen->font_man ,
@@ -428,6 +431,13 @@ draw_screen_vertical(
 		size_t  len ;
 		int  x ;
 
+	#ifdef  USE_FRAMEBUFFER
+		x_window_clear( &cand_screen->window , 0 ,
+				(xfont->height + LINE_SPACE) * cand_screen->num_per_window +
+					LINE_SPACE ,
+				win_width , xfont->height) ;
+	#endif
+
 		len = kik_snprintf( navi , MAX_NUM_OF_DIGITS * 2 + 2 , "%d/%d",
 				cand_screen->index + 1 ,
 				cand_screen->num_of_candidates) ;
@@ -442,10 +452,9 @@ draw_screen_vertical(
 
 			ml_char_init( &ch) ;
 
-			ml_char_set( &ch , &navi[i] , 1 ,
-				     US_ASCII , 0 , 0 ,
+			ml_char_set( &ch , navi[i] , US_ASCII , 0 , 0 ,
 				     ML_FG_COLOR , ML_BG_COLOR ,
-				     0 , 0) ;
+				     0 , 0 , 0) ;
 
 			x_draw_str( &cand_screen->window ,
 				    cand_screen->font_man ,
@@ -533,8 +542,8 @@ draw_screen_horizontal(
 
 			ml_char_init( &ch) ;
 
-			ml_char_set( &ch , &digit[j] , 1 , US_ASCII , 0 , 0 ,
-				     ML_FG_COLOR , ML_BG_COLOR , 0 , 0) ;
+			ml_char_set( &ch , digit[j] , US_ASCII , 0 , 0 ,
+				     ML_FG_COLOR , ML_BG_COLOR , 0 , 0 , 0) ;
 
 			x_draw_str( &cand_screen->window ,
 				    cand_screen->font_man ,
@@ -690,13 +699,11 @@ hide(
 	x_im_candidate_screen_t *  cand_screen
 	)
 {
-	if( cand_screen->is_focused)
+	if( ! cand_screen->window.is_focused)
 	{
-		return  1 ;
+		x_window_unmap( &cand_screen->window) ;
 	}
 
-	x_window_unmap( &cand_screen->window) ;
-
 	return  1 ;
 }
 
@@ -831,7 +838,7 @@ set_candidate(
 
 	while( (*parser->next_char)( parser , &ch))
 	{
-		int  is_biwidth = 0 ;
+		int  is_fullwidth = 0 ;
 		int  is_comb = 0 ;
 
 		if( ml_convert_to_internal_ch( &ch ,
@@ -840,25 +847,23 @@ set_candidate(
 			continue ;
 		}
 
-		if( ch.property & MKF_BIWIDTH)
+		if( ch.property & MKF_FULLWIDTH)
 		{
-			is_biwidth = 1 ;
+			is_fullwidth = 1 ;
 		}
 		else if( ch.property & MKF_AWIDTH)
 		{
 			/* TODO: check col_size_of_width_a */
-			is_biwidth = 1 ;
+			is_fullwidth = 1 ;
 		}
 
 		if( ch.property & MKF_COMBINING)
 		{
 			is_comb = 1 ;
 
-			if( ml_char_combine( p - 1 ,
-					     ch.ch , ch.size , ch.cs ,
-					     is_biwidth , is_comb ,
-					     ML_FG_COLOR , ML_BG_COLOR ,
-					     0 , 0))
+			if( ml_char_combine( p - 1 , mkf_char_to_int(&ch) ,
+					ch.cs , is_fullwidth , is_comb ,
+					ML_FG_COLOR , ML_BG_COLOR , 0 , 0 , 0))
 			{
 				continue ;
 			}
@@ -868,10 +873,8 @@ set_candidate(
 			 */
 		}
 
-		ml_char_set( p , ch.ch , ch.size , ch.cs ,
-			     is_biwidth , is_comb ,
-			     ML_FG_COLOR , ML_BG_COLOR ,
-			     0 , 0) ;
+		ml_char_set( p , mkf_char_to_int(&ch) , ch.cs , is_fullwidth ,
+			is_comb , ML_FG_COLOR , ML_BG_COLOR , 0 , 0 , 0) ;
 
 		p++ ;
 		cand_screen->candidates[index].filled_len++ ;
@@ -967,22 +970,6 @@ window_exposed(
 				  win->height + MARGIN - 1);
 }
 
-static void
-window_focused(
-	x_window_t *  win
-	)
-{
-	((x_im_candidate_screen_t *) win)->is_focused = 1 ;
-}
-
-static void
-window_unfocused(
-	x_window_t *  win
-	)
-{
-	((x_im_candidate_screen_t *) win)->is_focused = 0 ;
-}
-
 static void
 button_pressed(
 	x_window_t *  win ,
@@ -1042,7 +1029,7 @@ x_im_candidate_screen_new(
 {
 	x_im_candidate_screen_t *  cand_screen ;
 
-	if( ( cand_screen = malloc( sizeof( x_im_candidate_screen_t))) == NULL)
+	if( ( cand_screen = calloc( 1 , sizeof( x_im_candidate_screen_t))) == NULL)
 	{
 	#ifdef  DEBUG
 		kik_warn_printf( KIK_DEBUG_TAG " malloc failed.\n") ;
@@ -1054,13 +1041,6 @@ x_im_candidate_screen_new(
 	cand_screen->font_man = font_man ;
 	cand_screen->color_man = color_man ;
 
-	cand_screen->candidates = NULL ;
-	cand_screen->num_of_candidates = 0 ;
-
-	cand_screen->index = 0 ;
-
-	cand_screen->is_focused = 0 ;
-
 	cand_screen->x = x ;
 	cand_screen->y = y ;
 	cand_screen->line_height = line_height_of_screen ;
@@ -1100,8 +1080,6 @@ x_im_candidate_screen_new(
 	cand_screen->window.window_finalized = window_finalized ;
 #endif
 	cand_screen->window.window_exposed = window_exposed ;
-	cand_screen->window.window_focused = window_focused ;
-	cand_screen->window.window_unfocused = window_unfocused ;
 #if  0
 	cand_screen->window.button_released = button_released ;
 #endif
@@ -1121,10 +1099,6 @@ x_im_candidate_screen_new(
 	cand_screen->set = set_candidate ;
 	cand_screen->select = select_candidate ;
 
-	/* callbacks for events of candidate_screen */
-	cand_screen->listener.self = NULL ;
-	cand_screen->listener.selected = NULL ;
-
 	if( ! x_display_show_root( disp , &cand_screen->window , x , y , XValue | YValue ,
 					  "mlterm-candidate-window" , 0))
 	{
