@@ -43,23 +43,34 @@ draw_screen(
 
 		width += x_calculate_char_width(
 					xfont ,
-					ml_char_bytes( &stat_screen->chars[i]) ,
-					ml_char_size( &stat_screen->chars[i]) ,
+					ml_char_code( &stat_screen->chars[i]) ,
 					ml_char_cs( &stat_screen->chars[i]) , NULL) ;
 	}
 
 	xfont = x_get_usascii_font( stat_screen->font_man) ;
 
-	x_window_resize( &stat_screen->window , width ,
-			 xfont->height + LINE_SPACE, 0) ;
-	/* Reset window position */
-	set_spot( stat_screen , stat_screen->x , stat_screen->y) ;
-#ifdef  USE_FRAMEBUFFER
-	x_window_clear_margin_area( &stat_screen->window) ;	/* XXX */
-	x_window_draw_rect_frame( &stat_screen->window , -MARGIN , -MARGIN ,
-				  stat_screen->window.width + MARGIN - 1 ,
-				  stat_screen->window.height + MARGIN - 1);
-#endif
+	if( x_window_resize( &stat_screen->window , width ,
+				xfont->height + LINE_SPACE, 0))
+	{
+		/* Reset window position */
+
+	#ifdef  USE_FRAMEBUFFER
+		if( set_spot( stat_screen , stat_screen->x , stat_screen->y))
+		{
+			/*
+			 * set_spot() -> x_window_move() -> window_exposed() ->
+			 * draw_screen() and x_window_draw_rect_frame().
+			 */
+			return ;
+		}
+
+		x_window_draw_rect_frame( &stat_screen->window , -MARGIN , -MARGIN ,
+					  stat_screen->window.width + MARGIN - 1 ,
+					  stat_screen->window.height + MARGIN - 1) ;
+	#else
+		set_spot( stat_screen , stat_screen->x , stat_screen->y) ;
+	#endif
+	}
 
 	x_draw_str_to_eol( &stat_screen->window ,
 			   stat_screen->font_man ,
@@ -111,13 +122,11 @@ hide(
 	x_im_status_screen_t *  stat_screen
 	)
 {
-	if( stat_screen->is_focused)
+	if( ! stat_screen->window.is_focused)
 	{
-		return  1 ;
+		x_window_unmap( &stat_screen->window) ;
 	}
 
-	x_window_unmap( &stat_screen->window) ;
-
 	return  1 ;
 }
 
@@ -155,14 +164,14 @@ set_spot(
 
 	if( stat_screen->window.x != x || stat_screen->window.y != y)
 	{
-	#ifdef  USE_FRAMEBUFFER
-		x_window_move_full_expose( &stat_screen->window , x , y) ;
-	#else
 		x_window_move( &stat_screen->window , x , y) ;
-	#endif
-	}
 
-	return  1 ;
+		return  1 ;
+	}
+	else
+	{
+		return  0 ;
+	}
 }
 
 static int
@@ -217,19 +226,19 @@ set(
 
 	while( (*parser->next_char)( parser , &ch))
 	{
-		int  is_biwidth = 0 ;
+		int  is_fullwidth = 0 ;
 		int  is_comb = 0 ;
 
 		if( ch.cs == ISO10646_UCS4_1)
 		{
-			if( ch.property & MKF_BIWIDTH)
+			if( ch.property & MKF_FULLWIDTH)
 			{
-				is_biwidth = 1 ;
+				is_fullwidth = 1 ;
 			}
 			else if( ch.property & MKF_AWIDTH)
 			{
 				/* TODO: check col_size_of_width_a */
-				is_biwidth = 1 ;
+				is_fullwidth = 1 ;
 			}
 		}
 
@@ -240,11 +249,9 @@ set(
 
 		if( is_comb)
 		{
-			if( ml_char_combine( p - 1 ,
-					     ch.ch , ch.size , ch.cs ,
-					     is_biwidth , is_comb ,
-					     ML_FG_COLOR , ML_BG_COLOR ,
-					     0 , 0))
+			if( ml_char_combine( p - 1 , mkf_char_to_int(&ch) ,
+				ch.cs , is_fullwidth , is_comb , ML_FG_COLOR ,
+				ML_BG_COLOR , 0 , 0 , 0))
 			{
 				continue;
 			}
@@ -259,10 +266,8 @@ set(
 			SET_MSB( ch.ch[0]) ;
 		}
 
-		ml_char_set( p , ch.ch , ch.size , ch.cs ,
-			     is_biwidth , is_comb ,
-			     ML_FG_COLOR , ML_BG_COLOR ,
-			     0 , 0) ;
+		ml_char_set( p , mkf_char_to_int(&ch) , ch.cs , is_fullwidth ,
+			is_comb , ML_FG_COLOR , ML_BG_COLOR , 0 , 0 , 0) ;
 
 		p++ ;
 		stat_screen->filled_len++ ;
@@ -313,22 +318,6 @@ window_exposed(
 				  win->height + MARGIN - 1);
 }
 
-static void
-window_focused(
-	x_window_t *  win
-	)
-{
-	((x_im_status_screen_t *) win)->is_focused = 1 ;
-}
-
-static void
-window_unfocused(
-	x_window_t *  win
-	)
-{
-	((x_im_status_screen_t *) win)->is_focused = 0 ;
-}
-
 
 /* --- global functions --- */
 
@@ -345,7 +334,7 @@ x_im_status_screen_new(
 {
 	x_im_status_screen_t *  stat_screen ;
 
-	if( ( stat_screen = malloc( sizeof( x_im_status_screen_t))) == NULL)
+	if( ( stat_screen = calloc( 1 , sizeof( x_im_status_screen_t))) == NULL)
 	{
 	#ifdef  DEBUG
 		kik_warn_printf( KIK_DEBUG_TAG " malloc failed.\n") ;
@@ -357,12 +346,6 @@ x_im_status_screen_new(
 	stat_screen->font_man = font_man ;
 	stat_screen->color_man = color_man ;
 
-	stat_screen->chars = NULL ;
-	stat_screen->num_of_chars = 0 ;
-	stat_screen->filled_len = 0 ;
-
-	stat_screen->is_focused = 0 ;
-
 	stat_screen->x = x ;
 	stat_screen->y = y ;
 	stat_screen->line_height = line_height ;
@@ -398,8 +381,6 @@ x_im_status_screen_new(
 	stat_screen->window.window_finalized = window_finalized ;
 #endif
 	stat_screen->window.window_exposed = window_exposed ;
-	stat_screen->window.window_focused = window_focused ;
-	stat_screen->window.window_unfocused = window_unfocused ;
 
 	/* methods of x_im_status_screen_t */
 	stat_screen->delete = delete ;
