@@ -219,10 +219,8 @@ x_prepare_for_main_config(
 		"connecting server by default") ;
 #endif
 #ifdef  USE_LIBSSH2
-	#ifdef  USE_WIN32API
-	kik_conf_add_opt( conf , '\0' , "nodialog" , 1 , "skip_dialog" ,
-		"if possible, skip dialog to input server address, password and so on") ;
-	#endif
+	kik_conf_add_opt( conf , '\0' , "dialog" , 1 , "always_show_dialog" ,
+		"always show dialog to input server address, password and so on [false]") ;
 	kik_conf_add_opt( conf , '\0' , "pubkey" , 0 , "ssh_public_key" ,
 		"ssh public key file "
 	#ifdef  USE_WIN32API
@@ -243,6 +241,7 @@ x_prepare_for_main_config(
 		"preferred cipher list") ;
 	kik_conf_add_opt( conf , '\0' , "x11" , 1 , "ssh_x11_forwarding" ,
 		"allow x11 forwarding [false]") ;
+	kik_conf_add_opt( conf , '\0' , "scp" , 1 , "allow_scp" , "allow scp [false]") ;
 #endif
 	kik_conf_add_opt( conf , '\0' , "csp" , 0 , "letter_space" ,
 		"extra space between letters in pixels [0]") ;
@@ -272,6 +271,13 @@ x_prepare_for_main_config(
 		"force unicode or decsp font for box-drawing characters. [noconv]") ;
 	kik_conf_add_opt( conf , '\0' , "urgent" , 1 , "use_urgent_bell" ,
 		"draw the user's attention when making a bell sound. [false]") ;
+	kik_conf_add_opt( conf , '\0' , "locale" , 0 , "locale" , "set locale.") ;
+	kik_conf_add_opt( conf , '\0' , "ucsnoconv" , 0 , "unicode_noconv_areas" ,
+		"use unicode fonts partially regardless of -n option.") ;
+#ifdef  USE_GRF
+	kik_conf_add_opt( conf , '\0' , "multivram" , 1 , "separate_wall_picture" ,
+		"draw wall picture on another vram. (available on 4bpp) [false]") ;
+#endif
 #ifdef  USE_IM_CURSOR_COLOR
 	kik_conf_add_opt( conf , '\0' , "imcolor" , 0 , "im_cursor_color" ,
 		"cursor color when input method is activated. [false]") ;
@@ -293,6 +299,13 @@ x_main_config_init(
 	char *  value ;
 	char *  invalid_msg = "%s %s is not valid.\n" ;
 
+	memset( main_config , 0 , sizeof(x_main_config_t)) ;
+
+	if( ( value = kik_conf_get_value( conf , "locale")))
+	{
+		kik_locale_init( value) ;
+	}
+
 #if ! defined(USE_WIN32GUI) && ! defined(USE_FRAMEBUFFER)
 	if( ( value = kik_conf_get_value( conf , "display")) == NULL)
 #endif
@@ -332,22 +345,16 @@ x_main_config_init(
 		main_config->font_size = x_get_min_font_size() ;
 	}
 
-	main_config->app_name = NULL ;
-
 	if( ( value = kik_conf_get_value( conf , "app_name")))
 	{
 		main_config->app_name = strdup( value) ;
 	}
-
-	main_config->title = NULL ;
 	
 	if( ( value = kik_conf_get_value( conf , "title")))
 	{
 		main_config->title = strdup( value) ;
 	}
 
-	main_config->icon_name = NULL ;
-
 #if ! defined(USE_WIN32GUI) && ! defined(USE_FRAMEBUFFER)
 	if( ( value = kik_conf_get_value( conf , "icon_name")))
 	{
@@ -357,8 +364,6 @@ x_main_config_init(
 
 	/* BACKWARD COMPAT (3.1.7 or before) */
 #if  1
-	memset( main_config->shortcut_strs , 0 , sizeof(main_config->shortcut_strs)) ;
-
 	if( ( value = kik_conf_get_value( conf , "conf_menu_path_1")))
 	{
 		main_config->shortcut_strs[0] = malloc( 5 + strlen(value) + 2 + 1) ;
@@ -386,9 +391,6 @@ x_main_config_init(
 		sprintf( main_config->shortcut_strs[3] , "\"exesel:%s\"" , value) ;
 	}
 #endif
-
-	/* use default value */
-	main_config->scrollbar_view_name = NULL ;
 	
 	if( ( value = kik_conf_get_value( conf , "scrollbar_view_name")))
 	{
@@ -405,8 +407,6 @@ x_main_config_init(
 		}
 	}
 
-	main_config->use_dynamic_comb = 0 ;
-
 	if( ( value = kik_conf_get_value( conf , "use_dynamic_comb")))
 	{
 		if( strcmp( value , "true") == 0)
@@ -415,8 +415,6 @@ x_main_config_init(
 		}
 	}
 
-	main_config->logging_vt_seq = 0 ;
-
 	if( ( value = kik_conf_get_value( conf , "logging_vt_seq")))
 	{
 		if( strcmp( value , "true") == 0)
@@ -464,8 +462,6 @@ x_main_config_init(
 		main_config->type_engine = x_get_type_engine_by_name( value) ;
 	}
 
-	main_config->font_present = 0 ;
-
 #if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
 	if( ( value = kik_conf_get_value( conf , "use_anti_alias")))
 	{
@@ -496,8 +492,6 @@ x_main_config_init(
 			main_config->font_present |= FONT_VAR_WIDTH ;
 		}
 	}
-
-	main_config->vertical_mode = 0 ;
 	
 	if( ( value = kik_conf_get_value( conf , "vertical_mode")))
 	{
@@ -507,56 +501,40 @@ x_main_config_init(
 		}
 	}
 
-	main_config->fg_color = NULL ;
-	
 	if( ( value = kik_conf_get_value( conf , "fg_color")))
 	{
 		main_config->fg_color = strdup( value) ;
 	}
-
-	main_config->bg_color = NULL ;
 	
 	if( ( value = kik_conf_get_value( conf , "bg_color")))
 	{
 		main_config->bg_color = strdup( value) ;
 	}
-
-	main_config->cursor_fg_color = NULL ;
 	
 	if( ( value = kik_conf_get_value( conf , "cursor_fg_color")))
 	{
 		main_config->cursor_fg_color = strdup( value) ;
 	}
-
-	main_config->cursor_bg_color = NULL ;
 	
 	if( ( value = kik_conf_get_value( conf , "cursor_bg_color")))
 	{
 		main_config->cursor_bg_color = strdup( value) ;
 	}
 
-	main_config->bd_color = NULL ;
-
 	if( ( value = kik_conf_get_value( conf , "bd_color")))
 	{
 		main_config->bd_color = strdup( value) ;
 	}
 
-	main_config->ul_color = NULL ;
-
 	if( ( value = kik_conf_get_value( conf , "ul_color")))
 	{
 		main_config->ul_color = strdup( value) ;
 	}
-
-	main_config->sb_fg_color = NULL ;
 	
 	if( ( value = kik_conf_get_value( conf , "sb_fg_color")))
 	{
 		main_config->sb_fg_color = strdup( value) ;
 	}
-
-	main_config->sb_bg_color = NULL ;
 	
 	if( ( value = kik_conf_get_value( conf , "sb_bg_color")))
 	{
@@ -571,13 +549,20 @@ x_main_config_init(
 	{
 		main_config->term_type = strdup( "xterm") ;
 	}
-	
-	main_config->x = 0 ;
-	main_config->y = 0 ;
+
+#ifdef  USE_FRAMEBUFFER
+	/*
+	 * The pty is always resized to fit the display size.
+	 * Don't use 80x24 for the default value because the screen is not drawn
+	 * correctly on startup if the display size / the us-ascii font size is
+	 * 80x24 by chance.
+	 */
+	main_config->cols = 1 ;
+	main_config->rows = 1 ;
+#else
 	main_config->cols = 80 ;
 	main_config->rows = 24 ;
-	main_config->geom_hint = 0 ;
-	
+
 	if( ( value = kik_conf_get_value( conf , "geometry")))
 	{
 		/*
@@ -596,6 +581,7 @@ x_main_config_init(
 			main_config->rows = 24 ;
 		}
 	}
+#endif
 
 	main_config->screen_width_ratio = 100 ;
 	
@@ -639,8 +625,6 @@ x_main_config_init(
 		}
 	}
 
-	main_config->line_space = 0 ;
-
 	if( ( value = kik_conf_get_value( conf , "line_space")))
 	{
 		u_int  size ;
@@ -654,8 +638,6 @@ x_main_config_init(
 			kik_msg_printf( invalid_msg , "line space" , value) ;
 		}
 	}
-
-	main_config->letter_space = 0 ;
 	
 	if( ( value = kik_conf_get_value( conf , "letter_space")))
 	{
@@ -671,29 +653,37 @@ x_main_config_init(
 		}
 	}
 
-	if( ( value = kik_conf_get_value( conf , "logsize")) == NULL)
+	main_config->num_of_log_lines = 128 ;
+
+	if( ( value = kik_conf_get_value( conf , "logsize")))
 	{
-		main_config->num_of_log_lines = 128 ;
-	}
-	else if( ! kik_str_to_uint( &main_config->num_of_log_lines , value))
-	{
-		kik_msg_printf( invalid_msg , "log size" , value) ;
+		u_int  size ;
+
+		if( kik_str_to_uint( &size , value))
+		{
+			main_config->num_of_log_lines = size ;
+		}
+		else
+		{
+			kik_msg_printf( invalid_msg , "log size" , value) ;
+		}
 
-		/* default value is used. */
-		main_config->num_of_log_lines = 128 ;
 	}
 
-	if( ( value = kik_conf_get_value( conf , "tabsize")) == NULL)
-	{
-		/* default value is used. */
-		main_config->tab_size = 8 ;
-	}
-	else if( ! kik_str_to_uint( &main_config->tab_size , value))
-	{
-		kik_msg_printf( invalid_msg , "tab size" , value) ;
+	main_config->tab_size = 8 ;
 
-		/* default value is used. */
-		main_config->tab_size = 8 ;
+	if( ( value = kik_conf_get_value( conf , "tabsize")))
+	{
+		u_int  size ;
+
+		if( kik_str_to_uint( &size , value))
+		{
+			main_config->tab_size = size ;
+		}
+		else
+		{
+			kik_msg_printf( invalid_msg , "tab size" , value) ;
+		}
 	}
 	
 	main_config->use_login_shell = 0 ;
@@ -706,8 +696,6 @@ x_main_config_init(
 		}
 	}
 
-	main_config->big5_buggy = 0 ;
-
 	if( ( value = kik_conf_get_value( conf , "big5_buggy")))
 	{
 		if( strcmp( value , "true") == 0)
@@ -735,8 +723,6 @@ x_main_config_init(
 		main_config->sb_mode = SBM_LEFT ;
 	}
 
-	main_config->iso88591_font_for_usascii = 0 ;
-
 	if( ( value = kik_conf_get_value( conf , "iso88591_font_for_usascii")))
 	{
 		if( strcmp( value , "true") == 0)
@@ -745,8 +731,6 @@ x_main_config_init(
 		}
 	}
 
-	main_config->unicode_policy = 0 ;
-
 	if( ( value = kik_conf_get_value( conf , "not_use_unicode_font")))
 	{
 		if( strcmp( value , "true") == 0)
@@ -755,6 +739,11 @@ x_main_config_init(
 		}
 	}
 
+	if( ( value = kik_conf_get_value( conf , "unicode_noconv_areas")))
+	{
+		ml_set_unicode_noconv_areas( value) ;
+	}
+
 	if( ( value = kik_conf_get_value( conf , "only_use_unicode_font")))
 	{
 		if( strcmp( value , "true") == 0)
@@ -804,8 +793,6 @@ x_main_config_init(
 		}
 	}
 
-	main_config->receive_string_via_ucs = 0 ;
-
 	if( ( value = kik_conf_get_value( conf , "receive_string_via_ucs")))
 	{
 		if( strcmp( value , "true") == 0)
@@ -838,8 +825,6 @@ x_main_config_init(
 		}
 	}
 
-	main_config->pic_file_path = NULL ;
-
 	if( ( value = kik_conf_get_value( conf , "wall_picture")))
 	{
 		if( *value != '\0')
@@ -848,8 +833,6 @@ x_main_config_init(
 		}
 	}
 
-	main_config->use_transbg = 0 ;
-
 	if( ( value = kik_conf_get_value( conf , "use_transbg")))
 	{
 		if( strcmp( value , "true") == 0)
@@ -947,8 +930,7 @@ x_main_config_init(
 		}
 	}
 
-	main_config->is_auto_encoding = 0 ;
-	if( ( value = kik_conf_get_value( conf , "ENCODING")))
+	if( ( value = kik_conf_get_value( conf , "encoding")))
 	{
 		while( ( main_config->encoding = ml_get_char_encoding( value))
 			== ML_UNKNOWN_ENCODING)
@@ -986,8 +968,6 @@ x_main_config_init(
 			main_config->use_bidi = 0 ;
 		}
 	}
-#else
-	main_config->use_bidi = 0 ;
 #endif
 
 	main_config->bidi_mode = BIDI_NORMAL_MODE ;
@@ -999,8 +979,6 @@ x_main_config_init(
 	}
 #endif
 
-	main_config->use_ind = 0 ;
-
 #if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_IND)
 	if( ( value = kik_conf_get_value( conf , "use_ind")))
 	{
@@ -1017,10 +995,6 @@ x_main_config_init(
 	{
 		main_config->mod_meta_key = strdup( value) ;
 	}
-	else
-	{
-		main_config->mod_meta_key = NULL ;
-	}
 	
 	if( ( value = kik_conf_get_value( conf , "mod_meta_mode")))
 	{
@@ -1048,8 +1022,6 @@ x_main_config_init(
 		}
 	}
 
-	main_config->use_vertical_cursor = 0 ;
-
 	if( ( value = kik_conf_get_value( conf , "use_vertical_cursor")))
 	{
 		if( strcmp( value , "true") == 0)
@@ -1058,8 +1030,6 @@ x_main_config_init(
 		}
 	}
 	
-	main_config->use_extended_scroll_shortcut = 0 ;
-	
 	if( ( value = kik_conf_get_value( conf , "use_extended_scroll_shortcut")))
 	{
 		if( strcmp( value , "true") == 0)
@@ -1067,8 +1037,6 @@ x_main_config_init(
 			main_config->use_extended_scroll_shortcut = 1 ;
 		}
 	}
-
-	main_config->borderless = 0 ;
 	
 	if( ( value = kik_conf_get_value( conf , "borderless")))
 	{
@@ -1102,10 +1070,6 @@ x_main_config_init(
 	{
 		main_config->icon_path = strdup( value) ;
 	}
-	else
-	{
-		main_config->icon_path = NULL ;
-	}
 
 	if( ( value = kik_conf_get_value( conf , "input_method")))
 	{
@@ -1168,12 +1132,6 @@ x_main_config_init(
 			*p2 = '\0' ;
 		}
 	}
-	else
-	{
-		main_config->init_str = NULL ;
-	}
-
-	main_config->parent_window = 0 ;
 	
 	if( ( value = kik_conf_get_value( conf , "parent_window")))
 	{
@@ -1190,8 +1148,6 @@ x_main_config_init(
 	}
 	
 #if  defined(USE_WIN32API) || defined(USE_LIBSSH2)
-	main_config->server_list = NULL ;
-	
 	if( ( value = kik_conf_get_value( conf , "server_list")))
 	{
 		if( ( main_config->server_list = malloc( sizeof( char*) *
@@ -1220,8 +1176,6 @@ x_main_config_init(
 		}
 	}
 
-	main_config->default_server = NULL ;
-	
 	if( ( value = kik_conf_get_value( conf , "default_server")))
 	{
 		if( *value && ( main_config->default_server = strdup( value)))
@@ -1232,30 +1186,22 @@ x_main_config_init(
 		#endif
 		}
 	}
-#endif
 
-#if  defined(USE_WIN32API) && defined(USE_LIBSSH2)
-	main_config->skip_dialog = 0 ;
-	
-	if( ( value = kik_conf_get_value( conf , "skip_dialog")))
+	if( ( value = kik_conf_get_value( conf , "always_show_dialog")))
 	{
 		if( strcmp( value , "true") == 0)
 		{
-			main_config->skip_dialog = 1 ;
+			main_config->show_dialog = 1 ;
 		}
 	}
 #endif
 
 #ifdef  USE_LIBSSH2
-	main_config->public_key = NULL ;
-
 	if( ( value = kik_conf_get_value( conf , "ssh_public_key")))
 	{
 		main_config->public_key = strdup( value) ;
 	}
 
-	main_config->private_key = NULL ;
-
 	if( ( value = kik_conf_get_value( conf , "ssh_private_key")))
 	{
 		main_config->private_key = strdup( value) ;
@@ -1268,12 +1214,18 @@ x_main_config_init(
 
 	if( ( value = kik_conf_get_value( conf , "ssh_x11_forwarding")))
 	{
-		ml_pty_ssh_set_use_x11_forwarding( ( strcmp( value , "true") == 0)) ;
+		main_config->use_x11_forwarding = (strcmp( value , "true") == 0) ;
+	}
+
+	if( ( value = kik_conf_get_value( conf , "allow_scp")))
+	{
+		if( strcmp( value , "true") == 0)
+		{
+			ml_set_use_scp( 1) ;
+		}
 	}
 #endif
 
-	main_config->allow_osc52 = 0 ;
-
 	if( ( value = kik_conf_get_value( conf , "allow_osc52")))
 	{
 		if( strcmp( value , "true") == 0)
@@ -1282,8 +1234,6 @@ x_main_config_init(
 		}
 	}
 
-	main_config->blink_cursor = 0 ;
-
 	if( ( value = kik_conf_get_value( conf , "blink_cursor")))
 	{
 		if( strcmp( value , "true") == 0)
@@ -1298,7 +1248,8 @@ x_main_config_init(
 	{
 		u_int  margin ;
 
-		if( kik_str_to_uint( &margin , value) && margin <= 255)
+		/* 640x480 => (640-224*2)x(480-224*2) => 192x32 on framebuffer. */
+		if( kik_str_to_uint( &margin , value) && margin <= 224)
 		{
 			main_config->margin = margin ;
 		}
@@ -1314,8 +1265,6 @@ x_main_config_init(
 		}
 	}
 
-	main_config->hide_underline = 0 ;
-
 	if( ( value = kik_conf_get_value( conf , "hide_underline")))
 	{
 		if( strcmp( value , "true") == 0)
@@ -1343,8 +1292,6 @@ x_main_config_init(
 		ml_set_auto_restart_cmd( kik_get_prog_path()) ;
 	}
 
-	main_config->use_local_echo = 0 ;
-
 	if( ( value = kik_conf_get_value( conf , "use_local_echo")))
 	{
 		if( strcmp( value , "true") == 0)
@@ -1383,6 +1330,18 @@ x_main_config_init(
 		}
 	}
 
+#ifdef  USE_GRF
+	if( ( value = kik_conf_get_value( conf , "separate_wall_picture")))
+	{
+		if( strcmp( value , "true") == 0)
+		{
+			extern int  separate_wall_picture ;
+
+			separate_wall_picture = 1 ;
+		}
+	}
+#endif
+
 #ifdef  USE_IM_CURSOR_COLOR
 	if( ( value = kik_conf_get_value( conf , "im_cursor_color")))
 	{
@@ -1395,16 +1354,7 @@ x_main_config_init(
 
 	if( ( value = kik_conf_get_value( conf , "exec_cmd")) && strcmp( value , "true") == 0)
 	{
-		if( ( main_config->cmd_argv = malloc( sizeof( char*) * (argc + 1))) == NULL)
-		{
-		#ifdef  DEBUG
-			kik_warn_printf( KIK_DEBUG_TAG " malloc() failed.\n") ;
-		#endif
-		
-			main_config->cmd_path = NULL ;
-			main_config->cmd_argv = NULL ;
-		}
-		else
+		if( ( main_config->cmd_argv = malloc( sizeof( char*) * (argc + 1))))
 		{
 			/*
 			 * !! Notice !!
@@ -1418,11 +1368,6 @@ x_main_config_init(
 			main_config->cmd_argv[argc] = NULL ;
 		}
 	}
-	else
-	{
-		main_config->cmd_path = NULL ;
-		main_config->cmd_argv = NULL ;
-	}
 
 	return  1 ;
 }
@@ -1482,13 +1427,11 @@ x_main_config_add_to_server_list(
 	)
 {
 	u_int  nlist ;
-	int  found ;
 	size_t  len ;
 	size_t  add_len ;
 	char *  p ;
 	char **  pp ;
 
-	found = 0 ;
 	nlist = 0 ;
 	len = 0 ;
 	if( main_config->server_list)
