@@ -102,12 +102,13 @@ typedef struct x_main_config
 	int8_t  logging_vt_seq ;
 	int8_t  allow_osc52 ;
 	int8_t  blink_cursor ;
-	int8_t  margin ;
+	u_int8_t  margin ;
 	int8_t  hide_underline ;
 	int8_t  use_bold_font ;
 	int8_t  use_local_echo ;
-#if  defined(USE_WIN32API) && defined(USE_LIBSSH2)
-	int8_t  skip_dialog ;
+	int8_t  use_x11_forwarding ;
+#if  defined(USE_WIN32API) || defined(USE_LIBSSH2)
+	int8_t  show_dialog ;
 #endif
 
 } x_main_config_t ;
