@@ -134,18 +134,6 @@ typedef struct  x_screen
 
 	x_icon_picture_t *  icon ;
 
-#ifdef  ENABLE_SIXEL
-	x_picture_manager_t *  pic_man ;
-#endif
-
-	/*
-	 * These members mustn't be changed by mlterm configuration protocol,
-	 * since they can be executed.
-	 */
-	char *  conf_menu_path_1 ;
-	char *  conf_menu_path_2 ;
-	char *  conf_menu_path_3 ;
-
 	char  prev_mouse_report_seq[5] ;
 
 	u_int8_t  fade_ratio ;
@@ -157,14 +145,18 @@ typedef struct  x_screen
 	int8_t  font_or_color_config_updated ;	/* 0x1 = font updated, 0x2 = color updated */
 	int8_t  cursor_blink_wait ;
 	int8_t  hide_underline ;
+	int8_t  processing_vtseq ;
 
 } x_screen_t ;
 
 
-/* xterm = traditional, menuN = conf_menu_path_N */
-int  x_set_button3_behavior( const char *  mode) ;
+void  x_exit_backscroll_by_pty( int  flag) ;
 
-int  x_set_im_cursor_color( char *  color) ;
+void  x_allow_change_shortcut( int  flag) ;
+
+#ifdef  USE_IM_CURSOR_COLOR
+void  x_set_im_cursor_color( char *  color) ;
+#endif
 
 x_screen_t *  x_screen_new( ml_term_t *  term , x_font_manager_t *  font_man ,
 	x_color_manager_t *  color_man , x_termcap_entry_t *  termcap ,
@@ -174,7 +166,6 @@ x_screen_t *  x_screen_new( ml_term_t *  term , x_font_manager_t *  font_man ,
 	char *  mod_meta_key , x_mod_meta_mode_t  mod_meta_mode ,
 	x_bel_mode_t  bel_mode , int  receive_string_via_ucs , char *  pic_file_path ,
 	int  use_transbg , int  use_vertical_cursor , int  big5_buggy ,
-	char *  conf_menu_path_1 , char *  conf_menu_path_2 , char *  conf_menu_path_3 ,
 	int  use_extended_scroll_shortcut , int  borderless , u_int  line_space ,
 	char *  input_method , int  allow_osc52 , int  blink_cursor , int  margin ,
 	int  hide_underline) ;
