@@ -114,6 +114,8 @@ color_config_updated(void)
 	
 	x_color_cache_unload_all() ;
 
+	x_display_reset_cmap() ;
+
 	for( count = 0 ; count < num_of_screens ; count++)
 	{
 		x_screen_reset_view( screens[count]) ;
@@ -212,33 +214,47 @@ open_pty_intern(
 		char *  port ;
 		char *  encoding ;
 		char *  exec_cmd ;
+		int  x11_fwd ;
+		void *  session ;
+
+		x11_fwd = main_config.use_x11_forwarding ;
 
 	#ifdef  USE_LIBSSH2
-		if(
+		if( ! main_config.show_dialog &&
 		#ifdef  USE_WIN32API
-		    main_config.skip_dialog && main_config.default_server &&
+		    main_config.default_server &&
 		#endif
 		    kik_parse_uri( NULL , &user , &host , &port , NULL , &encoding ,
 			kik_str_alloca_dup( main_config.default_server)) &&
-		    ml_search_ssh_session( host , port , user))
+		    ( session = ml_search_ssh_session( host , port , user)))
 		{
 			uri = strdup( main_config.default_server) ;
 			pass = strdup( "") ;
 			exec_cmd = NULL ;
+
+			ml_pty_ssh_set_use_x11_forwarding( session , x11_fwd) ;
 		}
 		else
 	#endif
-		if( ! x_connect_dialog( &uri , &pass , &exec_cmd , display , window ,
+		if( ! x_connect_dialog( &uri , &pass , &exec_cmd , &x11_fwd , display , window ,
 				main_config.server_list , main_config.default_server))
 		{
 			kik_msg_printf( "Connect dialog is canceled.\n") ;
 
 			return  0 ;
 		}
-		else if( ! kik_parse_uri( NULL , NULL , NULL , NULL , NULL , &encoding ,
-				kik_str_alloca_dup( uri)) )
+		else
 		{
-			encoding = NULL ;
+			if( ! kik_parse_uri( NULL , &user , &host , &port , NULL , &encoding ,
+					kik_str_alloca_dup( uri)) )
+			{
+				encoding = NULL ;
+			}
+
+		#ifdef  USE_LIBSSH2
+			ml_pty_ssh_set_use_x11_forwarding(
+				ml_search_ssh_session( host , port , user) , x11_fwd) ;
+		#endif
 		}
 
 	#ifdef  __DEBUG
@@ -368,7 +384,7 @@ open_pty_intern(
 #if  defined(USE_WIN32API) || defined(USE_LIBSSH2)
 	if( uri)
 	{
-		if( ret)
+		if( ret && kik_compare_str( uri , main_config.default_server) != 0)
 		{
 			x_main_config_add_to_server_list( &main_config , uri) ;
 			free( main_config.default_server) ;
@@ -450,7 +466,7 @@ open_screen_intern(
 	}
 	else
 	{
-	#if  0
+	#ifdef  __ANDROID__
 		if( ( term = ml_get_detached_term( NULL)) == NULL &&
 			( term = create_term_intern()) == NULL)
 	#else
@@ -491,41 +507,13 @@ open_screen_intern(
 		main_config.step_in_changing_font_size ,
 		main_config.letter_space , main_config.use_bold_font)) == NULL)
 	{
-		char **  names ;
+		char *  name ;
 
-		names = x_font_get_encoding_names( usascii_font_cs) ;
-		if( names == NULL || names[0] == NULL)
-		{
-			kik_msg_printf(
-			  "Current encoding \"%s\" is supported only with "
-			  "Unicode font.  "
-			  "Please use \"-u\" option.\n",
-			  ml_get_char_encoding_name(ml_term_get_encoding(term))) ;
-		}
-		else if( strcmp( names[0] , "iso10646-1") == 0)
-		{
-			kik_msg_printf(
-			  "No fonts found for charset \"%s\".  "
-			  "Please install fonts.\n" , names[0]) ;
-		}
-		else if( names[1] == NULL)
-		{
-			/*
-			 * names[0] exists but x_font_manager_new failed.
-			 * It means that no fonts for names[0] exit in your system.
-			 */
-			kik_msg_printf(
-			  "No fonts found for charset \"%s\".  "
-			  "Please install fonts or use Unicode font "
-			  "(\"-u\" option).\n" , names[0]) ;
-		}
-		else
-		{
-			kik_msg_printf(
-			  "No fonts found for charset \"%s\" or \"%s\".  "
-			  "Please install fonts or use Unicode font "
-			  "(\"-u\" option).\n" , names[0] , names[1] );
-		}
+		name = x_get_charset_name( usascii_font_cs) ;
+
+		kik_msg_printf( "No fonts found for %s. Please install fonts "
+			"and edit the font config file in ~/.mlterm.\n" ,
+			name ? name : "US-ASCII") ;
 
 	#ifdef  DEBUG
 		kik_warn_printf( KIK_DEBUG_TAG " x_font_manager_new() failed.\n") ;
@@ -572,23 +560,17 @@ open_screen_intern(
 		goto  error ;
 	}
 
-	if( main_config.use_scrollbar)
+#ifndef  __ANDROID__
+	if( main_config.use_scrollbar &&
+	    ( sb_screen = x_sb_screen_new( screen ,
+				main_config.scrollbar_view_name ,
+				main_config.sb_fg_color , main_config.sb_bg_color ,
+				main_config.sb_mode)))
 	{
-		if( ( sb_screen = x_sb_screen_new( screen ,
-					main_config.scrollbar_view_name ,
-					main_config.sb_fg_color , main_config.sb_bg_color ,
-					main_config.sb_mode)) == NULL)
-		{
-		#ifdef  DEBUG
-			kik_warn_printf( KIK_DEBUG_TAG " x_sb_screen_new() failed.\n") ;
-		#endif
-
-			goto  error ;
-		}
-
 		root = &sb_screen->window ;
 	}
 	else
+#endif
 	{
 		root = &screen->window ;
 	}
@@ -744,6 +726,12 @@ __exit(
 
 	main_loop_final() ;
 
+#if  defined(USE_WIN32API) && defined(USE_LIBSSH2)
+	WSACleanup() ;
+#endif
+
+	kik_dl_close_all() ;
+
 	kik_msg_printf( "reporting unfreed memories --->\n") ;
 
 	kik_alloca_garbage_collect() ;
@@ -771,15 +759,39 @@ open_pty(
 	}
 	else
 	{
+	#if  defined(USE_WIN32API) || defined(USE_LIBSSH2)
+		char *  default_server ;
+		int  show_dialog ;
+	#endif
+		int  ret ;
+
 		if( ( new = create_term_intern()) == NULL)
 		{
 			return ;
 		}
 
-		if( ! open_pty_intern( new , main_config.cmd_path ,
+	#if  defined(USE_WIN32API) || defined(USE_LIBSSH2)
+		default_server = main_config.default_server ;
+		main_config.default_server = ml_term_get_uri( screen->term) ;
+		/*
+		 * If show_dialog == 1, main_config.default_server can be
+		 * free'ed in open_pty_intern.
+		 */
+		show_dialog = main_config.show_dialog ;
+		main_config.show_dialog = 0 ;
+	#endif
+
+		ret = open_pty_intern( new , main_config.cmd_path ,
 			main_config.cmd_argv ,
 			DisplayString( screen->window.disp->display) ,
-			x_get_root_window( &screen->window)->my_window))
+			x_get_root_window( &screen->window)->my_window) ;
+
+	#if  defined(USE_WIN32API) || defined(USE_LIBSSH2)
+		main_config.default_server = default_server ;
+		main_config.show_dialog = show_dialog ;
+	#endif
+
+		if( ! ret)
 		{
 			ml_destroy_term( new) ;
 
@@ -928,10 +940,23 @@ open_screen(
 	)
 {
 	char *  disp_name ;
+#if  defined(USE_WIN32API) || defined(USE_LIBSSH2)
+	char *  default_server ;
+	int  show_dialog ;
+#endif
 
-	/* Saving default disp_name option. */
 	disp_name = main_config.disp_name ;
 	main_config.disp_name = cur_screen->window.disp->name ;
+#if  defined(USE_WIN32API) || defined(USE_LIBSSH2)
+	default_server = main_config.default_server ;
+	main_config.default_server = ml_term_get_uri( cur_screen->term) ;
+	/*
+	 * If show_dialog == 1, main_config.default_server can be
+	 * free'ed in open_pty_intern.
+	 */
+	show_dialog = main_config.show_dialog ;
+	main_config.show_dialog = 0 ;
+#endif
 
 	if( ! open_screen_intern( NULL))
 	{
@@ -940,8 +965,11 @@ open_screen(
 	#endif
 	}
 
-	/* Restoring default disp_name option. */
 	main_config.disp_name = disp_name ;
+#if  defined(USE_WIN32API) || defined(USE_LIBSSH2)
+	main_config.default_server = default_server ;
+	main_config.show_dialog = show_dialog ;
+#endif
 }
 
 static void
@@ -1057,6 +1085,9 @@ mlclient(
 		kik_conf_t *  conf ;
 		x_main_config_t  orig_conf ;
 		char *  pty ;
+	#if  defined(USE_WIN32API) || defined(USE_LIBSSH2)
+		char **  server_list ;
+	#endif
 		
 		if( argc >= 2 && *argv[1] != '-')
 		{
@@ -1091,6 +1122,10 @@ mlclient(
 		orig_conf = main_config ;
 
 		x_main_config_init( &main_config , conf , argc , argv) ;
+	#if  defined(USE_WIN32API) || defined(USE_LIBSSH2)
+		server_list = main_config.server_list ;
+		main_config.server_list = orig_conf.server_list ;
+	#endif
 
 		kik_conf_delete( conf) ;
 
@@ -1108,6 +1143,10 @@ mlclient(
 			}
 		}
 
+	#if  defined(USE_WIN32API) || defined(USE_LIBSSH2)
+		orig_conf.server_list = main_config.server_list ;
+		main_config.server_list = server_list ;
+	#endif
 		x_main_config_final( &main_config) ;
 
 		main_config = orig_conf ;
@@ -1262,7 +1301,6 @@ x_screen_manager_final(void)
 	}
 
 	free( screens) ;
-
 	free( dead_mask) ;
 
 	ml_term_manager_final() ;
@@ -1276,6 +1314,33 @@ x_screen_manager_final(void)
 	return  1 ;
 }
 
+#ifdef  __ANDROID__
+static int  suspended ;
+
+int
+x_screen_manager_suspend(void)
+{
+	u_int  count ;
+
+	x_close_dead_screens() ;
+
+	for( count = 0 ; count < num_of_screens ; count ++)
+	{
+		close_screen_intern( screens[count]) ;
+	}
+
+	free( screens) ;
+	screens = NULL ;
+	num_of_screens = 0 ;
+
+	x_display_close_all() ;
+
+	suspended = 1 ;
+
+	return  1 ;
+}
+#endif
+
 u_int
 x_screen_manager_startup(void)
 {
@@ -1284,6 +1349,14 @@ x_screen_manager_startup(void)
 
 	num_started = 0 ;
 
+#ifdef  __ANDROID__
+	if( suspended)
+	{
+		/* reload ~/.mlterm/main. */
+		config_saved() ;
+	}
+#endif
+
 	for( count = 0 ; count < num_of_startup_screens ; count ++)
 	{
 		if( ! open_screen_intern( NULL))
